<?php

namespace App\Exports;

use Maatwebsite\Excel\Concerns\FromCollection;
use Maatwebsite\Excel\Concerns\WithHeadings;
use Maatwebsite\Excel\Concerns\WithTitle;
use Maatwebsite\Excel\Concerns\WithStyles;
use Maatwebsite\Excel\Concerns\WithEvents;
use PhpOffice\PhpSpreadsheet\Worksheet\Worksheet;
use Maatwebsite\Excel\Events\AfterSheet;

use App\Models\Voucher;
use App\Models\Accounting;
use App\Models\Ledger;
use App\Models\Financialyear;

class LedgerExport implements FromCollection, WithHeadings, WithTitle, WithStyles, WithEvents
{
    protected $ledgerId;
    protected $startDate;
    protected $endDate;
    protected $accountId;
    protected $ledgerName;
    protected $ledgerDc;
    protected $closeBal;
        
    // Accept start and end dates in the constructor
    public function __construct($ledgerId, $startDate, $endDate, $accountId)
    {
        $this->ledgerId = $ledgerId;
        $this->startDate = $startDate;
        $this->endDate = $endDate;
        $this->accountId = $accountId;
        $this->closeBal = 0;
        
        $ledger = Ledger::find($this->ledgerId);
        $this->ledgerName = $ledger->ledger_name;
        $this->ledgerDc = $ledger->dc;
        
        $this->fyearId = Financialyear::getDefaultFyearId($this->accountId);
        $fyear = Financialyear::find($this->fyearId);
        if ($fyear) {
            $this->fsDate = $fyear->sdate;
            $this->feDate = $fyear->edate;
        } else {
            abort(404, 'Financial Year not found');
        }
    }
    
    /**
    * @return \Illuminate\Support\Collection
    */
    public function collection()
    {
        $open_bal = $this->getOpeningBalance();
        
        $vouchers = Voucher::whereNull('deleted_at')
                    ->where('vdate', '>=', $this->startDate)
                    ->where('vdate', '<=', $this->endDate)
                    ->where('account_id', $this->accountId)
                    ->where(function($query) {
                        $query->where('cr_ledger', $this->ledgerId)
                            ->orWhere('dr_ledger', $this->ledgerId);
                    })
                    ->get();
                    
        return $vouchers->map(function ($voucher) use (&$open_bal) {
            $dr=$cr=0; $ledger_name='';
            if($voucher->dr_ledger==$this->ledgerId){
                $dr = $voucher->vamount;
                $ledger_name = 'To '.$voucher->credit->ledger_name;
            }else if($voucher->cr_ledger==$this->ledgerId){
                $cr = $voucher->vamount;
                $ledger_name = 'By '.$voucher->debit->ledger_name;
            }
            if($this->ledgerDc=='dr'){
                $open_bal += $dr - $cr;
            }else{
                $open_bal += $cr - $dr;
            }
            
            $this->closeBal = $open_bal;
            
            return [
                $voucher->vdate_formatted,
                $ledger_name,
                $voucher->vno,
                $voucher->vtype->vtype_name,
                $dr,
                $cr,
                $open_bal
            ];
        });
    }
    
    public function headings(): array
    {
        return [
            'Date',
            'Particulars',
            'Voucher No',
            'Voucher Type',
            'Debit',
            'Credit',
            'Closing Balance'
        ];
    }
    
    public function title(): string
    {
        return 'Ledger Report - '.$this->ledgerName;
    }
        
    public function styles(Worksheet $sheet)
    {
        $sheet->getColumnDimension('A')->setWidth(15);
        $sheet->getColumnDimension('B')->setWidth(50);
        $sheet->getColumnDimension('C')->setWidth(15);
        $sheet->getColumnDimension('D')->setWidth(25);
        $sheet->getColumnDimension('E')->setWidth(20);
        $sheet->getColumnDimension('F')->setWidth(20);
        $sheet->getColumnDimension('G')->setWidth(20);
            
        $sheet->getStyle('A1:G1')->applyFromArray([
            'font' => [
                'bold' => true,
                'size' => 12,
            ],
            'alignment' => [
                'horizontal' => \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER,
                'vertical' => \PhpOffice\PhpSpreadsheet\Style\Alignment::VERTICAL_CENTER,
            ],
            'fill' => [
                'fillType' => \PhpOffice\PhpSpreadsheet\Style\Fill::FILL_SOLID,
                'startColor' => [
                    'rgb' => 'FFFF00',
                ]
            ]
        ]);
            
        $sheet->getStyle('A2:D' . $sheet->getHighestRow())->applyFromArray([
            'alignment' => [
                'horizontal' => \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER,
            ]
        ]);
            
        $sheet->getStyle('E2:G' . $sheet->getHighestRow())->applyFromArray([
            'numberFormat' => [
                'formatCode' => '0.00',
            ],
            'alignment' => [
                'horizontal' => \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_RIGHT,
            ]
        ]);
        
        return [];
    }

    public function registerEvents(): array
    {
        return [
            AfterSheet::class => function (AfterSheet $event) {
                $sheet = $event->sheet;

                // Insert the title row before the column headings (row 1)
                $sheet->insertNewRowBefore(1, 1); // Insert a new row at position 1 for the title
                $sheet->setCellValue('A1', 'Ledger Report of '.$this->ledgerName.' From '.date('d-m-Y',strtotime($this->startDate)).' To '.date('d-m-Y',strtotime($this->endDate)));
                $sheet->mergeCells('A1:G1');
                $sheet->getStyle('A1')->applyFromArray([
                    'font' => [
                        'bold' => true,
                        'size' => 14,
                    ],
                    'alignment' => [
                        'horizontal' => \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER,
                    ]
                ]);
                
                $dr=$cr=0;
                if($this->ledgerDc=='dr'){ $dr = $this->getOpeningBalance(); }
                else if($this->ledgerDc=='cr'){ $cr = $this->getOpeningBalance(); }
                
                // Insert opening balance row after the title and before the column headings
                $sheet->insertNewRowBefore(3, 1); // Insert one row at position 2 for opening balance
                $sheet->setCellValue('A3', '');
                $sheet->setCellValue('B3', 'Opening Balance');
                $sheet->setCellValue('C3', '');
                $sheet->setCellValue('D3', '');
                $sheet->setCellValueExplicit('E3', $dr, \PhpOffice\PhpSpreadsheet\Cell\DataType::TYPE_NUMERIC);
                $sheet->setCellValueExplicit('F3', $cr, \PhpOffice\PhpSpreadsheet\Cell\DataType::TYPE_NUMERIC);
                $sheet->setCellValueExplicit('G3', $this->getOpeningBalance(), \PhpOffice\PhpSpreadsheet\Cell\DataType::TYPE_NUMERIC);
                
                // Apply style for the opening balance row (row 3)
                $sheet->getStyle('A3:D3')->applyFromArray([
                    'font' => [
                        'bold' => false,
                    ],
                    'alignment' => [
                        'horizontal' => \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER,
                    ],
                    'fill' => [
                        'fillType' => \PhpOffice\PhpSpreadsheet\Style\Fill::FILL_SOLID,
                        'startColor' => [
                            'rgb' => 'FFFFFF',
                        ]
                    ]
                ]);
                    
                $sheet->getStyle('E3:G3')->applyFromArray([
                    'numberFormat' => [
                        'formatCode' => '0.00',
                    ],
                    'font' => [
                        'bold' => false,
                    ],
                    'alignment' => [
                        'horizontal' => \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_RIGHT,
                    ],
                    'fill' => [
                        'fillType' => \PhpOffice\PhpSpreadsheet\Style\Fill::FILL_SOLID,
                        'startColor' => [
                            'rgb' => 'FFFFFF',
                        ]
                    ]
                ]);

                // Add closing balance row after the last data entry
                $lastRow = $sheet->getHighestRow() + 1; // Get the last row after data
                $sheet->setCellValue('A' . $lastRow, '');
                $sheet->setCellValue('B' . $lastRow, 'Closing Balance');
                $sheet->setCellValue('C' . $lastRow, '');
                $sheet->setCellValue('D' . $lastRow, '');
                $sheet->setCellValue('E' . $lastRow, '');
                $sheet->setCellValue('F' . $lastRow, '');
                $sheet->setCellValueExplicit('G' . $lastRow, $this->closeBal, \PhpOffice\PhpSpreadsheet\Cell\DataType::TYPE_NUMERIC);
                
                // Apply style for the closing balance row (last row)
                $sheet->getStyle('A'.$lastRow.':D'.$lastRow)->applyFromArray([
                    'font' => [
                        'bold' => true,
                    ],
                    'alignment' => [
                        'horizontal' => \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER,
                    ]
                ]);
                    
                $sheet->getStyle('E'.$lastRow.':G'.$lastRow)->applyFromArray([
                    'numberFormat' => [
                        'formatCode' => '0.00',
                    ],
                    'font' => [
                        'bold' => true,
                    ],
                    'alignment' => [
                        'horizontal' => \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_RIGHT,
                    ]
                ]);
                
                $sheet->getStyle('A2:G'.$lastRow)->applyFromArray([
                    'font' => [
                        'name' => 'Calibri',
                        'size' => 12,
                    ],
                    'borders' => [
                        'allBorders' => [
                            'borderStyle' => \PhpOffice\PhpSpreadsheet\Style\Border::BORDER_THIN,
                            'color' => ['argb' => '000000'],
                        ],
                    ],
                ]);
                
                $sheet->getRowDimension(1)->setRowHeight(25);
                $sheet->getRowDimension(2)->setRowHeight(20);
            }
        ];
    }
    
    public function getOpeningBalance()
    {
        $pre_date = date('Y-m-d', (strtotime($this->startDate) - 86400));
                    
        $ledgerdet = Ledger::find($this->ledgerId);
        $pre_open_bal = Accounting::openingBalance($pre_date,$this->accountId,0,$this->ledgerId);
        $cur_open_bal = Accounting::openingBalance($this->endDate,$this->accountId,0,$this->ledgerId,$this->startDate);
        $ledger_open_bal = $pre_open_bal+$cur_open_bal;
                    
        if(in_array($ledgerdet->sub->acmaingrp_id, [3, 4, 5, 6, 10, 11])){
            $voucher_total = Accounting::ledgerTotal($this->ledgerId,$pre_date,$this->accountId,$this->fsDate);
        }else{
            $voucher_total = Accounting::ledgerTotal($this->ledgerId,$pre_date,$this->accountId);
        }
        
        if($ledgerdet->dc=='dr'){
            $voucher_open_bal = $voucher_total['dr_amount']-$voucher_total['cr_amount'];
        }else{
            $voucher_open_bal = $voucher_total['cr_amount']-$voucher_total['dr_amount'];
        }
        
        $open_bal = $ledger_open_bal+$voucher_open_bal;
        
        return $open_bal;
    }
}
