<?php

namespace App\Exports;

use Maatwebsite\Excel\Concerns\FromCollection;
use Maatwebsite\Excel\Concerns\WithHeadings;
use Maatwebsite\Excel\Concerns\WithTitle;
use Maatwebsite\Excel\Concerns\WithStyles;
use Maatwebsite\Excel\Concerns\WithEvents;
use PhpOffice\PhpSpreadsheet\Worksheet\Worksheet;
use Maatwebsite\Excel\Events\AfterSheet;

use App\Models\Library\LibraryBook;

class LibraryBooksExport implements FromCollection, WithHeadings, WithTitle, WithStyles, WithEvents
{
    protected $category;
    protected $subCategory;
    protected $location;
    protected $subLocation;
    protected $searchKey;
    
    public function __construct($category, $subCategory, $location, $subLocation, $searchKey)
    {
        $this->category = $category;
        $this->subCategory = $subCategory;
        $this->location = $location;
        $this->subLocation = $subLocation;
        $this->searchKey = $searchKey;
    }
    
    /**
    * @return \Illuminate\Support\Collection
    */
    public function collection()
    {
        $query = LibraryBook::query()
                    ->leftJoin('library_categories', 'library_books.category_id', '=', 'library_categories.id')
                    ->leftJoin('library_subcategories', 'library_books.subcategory_id', '=', 'library_subcategories.id')
                    ->leftJoin('library_locations', 'library_books.location_id', '=', 'library_locations.id')
                    ->leftJoin('library_sublocations', 'library_books.sublocation_id', '=', 'library_sublocations.id')
                    ->whereNull('library_books.deleted_at');
        if($this->category!=0){
            $query->where('library_books.category_id', $this->category);
        }
        if($this->subCategory!=0){
            $query->where('library_books.subcategory_id', $this->subCategory);
        }
        if($this->location!=0){
            $query->where('library_books.location_id', $this->location);
        }
        if($this->subLocation!=0){
            $query->where('library_books.sublocation_id', $this->subLocation);
        }
        if($this->searchKey!==null){
            $skey = $this->searchKey;
            $query->where(function ($qry) use ($skey) {
                $qry->where('library_books.book_no', 'like', "%$skey%")
                    ->orWhere('library_books.access_no', 'like', "%$skey%")
                    ->orWhere('library_books.title', 'like', "%$skey%")
                    ->orWhere('library_books.author', 'like', "%$skey%")
                    ->orWhere('library_books.subject', 'like', "%$skey%")
                    ->orWhere('library_books.keywords', 'like', "%$skey%")
                    ->orWhere('library_categories.name', 'like', "%$skey%")
                    ->orWhere('library_subcategories.name', 'like', "%$skey%")
                    ->orWhere('library_locations.name', 'like', "%$skey%")
                    ->orWhere('library_sublocations.name', 'like', "%$skey%");
            });
        }
        
        $books = $query->select(
                    'library_books.*',
                    'library_categories.name as category',
                    'library_subcategories.name as subcategory',
                    'library_locations.name as location',
                    'library_sublocations.name as sublocation'
                )->get();
        
        return $books->map(function ($book) {
            
            return [
                $book->book_no,
                $book->access_no,
                $book->title,
                $book->sub_title,
                $book->author,
                $book->category,
                $book->subcategory,
                $book->location,
                $book->sublocation,
                date('d/m/Y',strtotime($book->date)),
                $book->language,
                $book->publisher,
                $book->volume,
                $book->edition,
                $book->year,
                $book->price,
                $book->condition,
                $book->isbn,
                $book->reference,
                $book->awards,
                $book->keywords,
                $book->subject,
                $book->writer,
                $book->copies,
            ];
        });
    }
    
    public function headings(): array
    {
        return [
            'Book No',
            'Access No',
            'Title',
            'Sub Title',
            'Author',
            'Category',
            'Sub Category',
            'Location',
            'Sub Location',
            'Date',
            'Language',
            'Publisher',
            'Volume',
            'Edition',
            'Year',
            'Price',
            'Condition',
            'ISBN',
            'Reference',
            'Awards',
            'Keywords',
            'Subject',
            'Writer',
            'Copies',
        ];
    }
    
    public function title(): string
    {
        return 'Library Books';
    }
    
    public function styles(Worksheet $sheet)
    {
        $sheet->getColumnDimension('A')->setWidth(20);
        $sheet->getColumnDimension('B')->setWidth(20);
        $sheet->getColumnDimension('C')->setWidth(50);
        
        for ($column = ord('D'); $column <= ord('X'); $column++) {
            $columnLetter = chr($column);
            $sheet->getColumnDimension($columnLetter)->setWidth(25);
        }
            
        $sheet->getStyle('A1:X1')->applyFromArray([
            'font' => [
                'bold' => true,
                'size' => 12,
            ],
            'alignment' => [
                'horizontal' => \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER,
                'vertical' => \PhpOffice\PhpSpreadsheet\Style\Alignment::VERTICAL_CENTER,
            ],
            'fill' => [
                'fillType' => \PhpOffice\PhpSpreadsheet\Style\Fill::FILL_SOLID,
                'startColor' => [
                    'rgb' => 'FFFF00',
                ]
            ]
        ]);
            
        return [];
    }
    
    public function registerEvents(): array
    {
        return [
            AfterSheet::class => function (AfterSheet $event) {
                $sheet = $event->sheet;

                // Insert the title row before the column headings (row 1)
                $sheet->insertNewRowBefore(1, 1); // Insert a new row at position 1 for the title
                $sheet->setCellValue('A1', 'Library Books Report');
                $sheet->mergeCells('A1:G1');
                $sheet->getStyle('A1')->applyFromArray([
                    'font' => [
                        'bold' => true,
                        'size' => 14,
                    ],
                    'alignment' => [
                        'horizontal' => \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER,
                    ]
                ]);
                
                // Add closing balance row after the last data entry
                $lastRow = $sheet->getHighestRow(); // Get the last row after data
                $sheet->getStyle('A2:X'.$lastRow)->applyFromArray([
                    'font' => [
                        'name' => 'Calibri',
                        'size' => 12,
                    ],
                    'borders' => [
                        'allBorders' => [
                            'borderStyle' => \PhpOffice\PhpSpreadsheet\Style\Border::BORDER_THIN,
                            'color' => ['argb' => '000000'],
                        ],
                    ],
                    'alignment' => [
                        'horizontal' => \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER,
                        'vertical' => \PhpOffice\PhpSpreadsheet\Style\Alignment::VERTICAL_CENTER,
                    ],
                ]);
                
                $sheet->getRowDimension(1)->setRowHeight(25);
                $sheet->getRowDimension(2)->setRowHeight(20);
            }
        ];
    }
}
