<?php

namespace App\Exports;

use Maatwebsite\Excel\Concerns\FromCollection;
use Maatwebsite\Excel\Concerns\WithHeadings;
use Maatwebsite\Excel\Concerns\WithTitle;
use Maatwebsite\Excel\Concerns\WithStyles;
use Maatwebsite\Excel\Concerns\WithEvents;
use PhpOffice\PhpSpreadsheet\Worksheet\Worksheet;
use Maatwebsite\Excel\Events\AfterSheet;

use App\Models\Accounting;
use App\Models\Financialyear;

class ProfitLossExport implements FromCollection, WithHeadings, WithTitle, WithStyles, WithEvents
{
    protected $startDate;
    protected $endDate;
    protected $accountId;
    protected $fsDate;
    protected $feDate;
    protected $exTotal;
    protected $inTotal;
        
    // Accept start and end dates in the constructor
    public function __construct($startDate, $endDate, $accountId)
    {
        $this->startDate = $startDate;
        $this->endDate = $endDate;
        $this->accountId = $accountId;
        $this->exTotal = 0;
        $this->inTotal = 0;
        
        $this->fyearId = Financialyear::getDefaultFyearId($this->accountId);
        $fyear = Financialyear::find($this->fyearId);
        if($fyear){
            $this->fsDate = $fyear->sdate;
            $this->feDate = $fyear->edate;
        }
    }
    
    /**
    * @return \Illuminate\Support\Collection
    */
    public function collection()
    {
        $expense_count = Accounting::getLedgers("expense")->count();
        $expenses = Accounting::profitLoss("expense",$this->accountId,$this->startDate,$this->endDate,0,$expense_count,0,$this->fsDate);
            
        $income_count = Accounting::getLedgers("income")->count();
        $incomes = Accounting::profitLoss("income",$this->accountId,$this->startDate,$this->endDate,0,$income_count,0,$this->fsDate);
         
        // Initialize the result array
        $result = [];
            
        $exparray = $incarray = [];
        foreach($expenses as $expense){
            $exparray[] = $expense;
        }
        foreach($incomes as $income){
            $incarray[] = $income;
        }
            
        $total_rows = max(count($exparray), count($incarray));
            
        for ($i = 0; $i < $total_rows; $i++) {
            
            // Set default empty values if there's no corresponding entry
            $expense_ledger_name = isset($exparray[$i]) ? $exparray[$i]['ledger_name'] : '';
            $expense_amount = isset($exparray[$i]) ? $exparray[$i]['total_amount'] : '';
            $income_ledger_name = isset($incarray[$i]) ? $incarray[$i]['ledger_name'] : '';
            $income_amount = isset($incarray[$i]) ? $incarray[$i]['total_amount'] : '';
            
            if($expense_amount!=''){
                $this->exTotal += $expense_amount;
            }
            
            if($income_amount!=''){
                $this->inTotal += $income_amount;
            }
                
            // Add the row to the result array
            $result[] = [
                $expense_ledger_name,  // Column A: Ledger Name (Expense)
                $expense_amount,       // Column B: Total Amount (Expense)
                $income_ledger_name,   // Column C: Ledger Name (Income)
                $income_amount         // Column D: Total Amount (Income)
            ];
        }
            
        // Return the result as a collection to be used by Excel export
        return collect($result);
    }
        
    public function headings(): array
    {
        return [
            'Expense',
            'Amount',
            'Income',
            'Amount',
        ];
    }
    
    public function title(): string
    {
        return 'Profit & Loss';
    }
    
    public function styles(Worksheet $sheet)
    {
        $sheet->getColumnDimension('A')->setWidth(50);
        $sheet->getColumnDimension('B')->setWidth(20);
        $sheet->getColumnDimension('C')->setWidth(50);
        $sheet->getColumnDimension('D')->setWidth(20);
            
        $sheet->getStyle('A1:D1')->applyFromArray([
            'font' => [
                'bold' => true,
                'size' => 12,
            ],
            'alignment' => [
                'horizontal' => \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER,
                'vertical' => \PhpOffice\PhpSpreadsheet\Style\Alignment::VERTICAL_CENTER,
            ],
            'fill' => [
                'fillType' => \PhpOffice\PhpSpreadsheet\Style\Fill::FILL_SOLID,
                'startColor' => [
                    'rgb' => 'FFFF00',
                ]
            ]
        ]);
            
        return [];
    }
    
    public function registerEvents(): array
    {
        return [
            AfterSheet::class => function (AfterSheet $event) {
                $sheet = $event->sheet;
                    
                // Insert the title row before the column headings (row 1)
                $sheet->insertNewRowBefore(1, 1); // Insert a new row at position 1 for the title
                $sheet->setCellValue('A1', 'Profit & Loss From '.date('d-m-Y',strtotime($this->startDate)).' To '.date('d-m-Y',strtotime($this->endDate)));
                $sheet->mergeCells('A1:D1');
                $sheet->getStyle('A1')->applyFromArray([
                    'font' => [
                        'bold' => true,
                        'size' => 14,
                    ],
                    'alignment' => [
                        'horizontal' => \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER,
                    ]
                ]);
                
                $profit_loss = $this->inTotal-$this->exTotal;
                $exTitle=$exAmount=$inTitle=$inAmount='';
                if($profit_loss > 0){
                    $exTitle = 'Surplus (Profit)';
                    $exAmount = $profit_loss;
                    $this->exTotal += $profit_loss;
                }else if($profit_loss < 0){
                    $inTitle = 'Deficit (Loss)';
                    $inAmount = (-$profit_loss);
                    $this->inTotal += (-$profit_loss);
                }
                
                $lastRow = $sheet->getHighestRow() + 1; // Get the last row after data
                $sheet->setCellValue('A'.$lastRow, $exTitle);
                $sheet->setCellValue('B'.$lastRow, $exAmount);
                $sheet->setCellValue('C'.$lastRow, $inTitle);
                $sheet->setCellValue('D'.$lastRow, $inAmount);
                
                $lastRow++;
                $sheet->setCellValue('A'.$lastRow, 'TOTAL');
                $sheet->setCellValue('B'.$lastRow, $this->exTotal);
                $sheet->setCellValue('C'.$lastRow, 'TOTAL');
                $sheet->setCellValue('D'.$lastRow, $this->inTotal);
                
                $sheet->getStyle('A'.$lastRow.':D'.$lastRow)->applyFromArray([
                    'font' => [
                        'bold' => true,
                    ]
                ]);
                
                $sheet->getStyle('B2:B'.$lastRow)->applyFromArray([
                    'numberFormat' => [
                        'formatCode' => '0.00',
                    ],
                    'alignment' => [
                        'horizontal' => \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_RIGHT,
                    ]
                ]);
                
                $sheet->getStyle('D2:D'.$lastRow)->applyFromArray([
                    'numberFormat' => [
                        'formatCode' => '0.00',
                    ],
                    'alignment' => [
                        'horizontal' => \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_RIGHT,
                    ]
                ]);
                
                $sheet->getStyle('A2:D'.$lastRow)->applyFromArray([
                    'font' => [
                        'name' => 'Calibri',
                        'size' => 12,
                    ],
                    'borders' => [
                        'allBorders' => [
                            'borderStyle' => \PhpOffice\PhpSpreadsheet\Style\Border::BORDER_THIN,
                            'color' => ['argb' => '000000'],
                        ],
                    ],
                ]);
                    
                $sheet->getRowDimension(1)->setRowHeight(25);
                $sheet->getRowDimension(2)->setRowHeight(20);
                
                $sheet->getStyle('A1:D'.$lastRow)->applyFromArray([
                    'alignment' => [
                        'vertical' => \PhpOffice\PhpSpreadsheet\Style\Alignment::VERTICAL_CENTER,
                    ]
                ]);
            }
        ];
    }
}
