<?php

namespace App\Exports;

use Maatwebsite\Excel\Concerns\FromCollection;
use Maatwebsite\Excel\Concerns\WithHeadings;
use Maatwebsite\Excel\Concerns\WithTitle;
use Maatwebsite\Excel\Concerns\WithStyles;
use Maatwebsite\Excel\Concerns\WithEvents;
use PhpOffice\PhpSpreadsheet\Worksheet\Worksheet;
use Maatwebsite\Excel\Events\AfterSheet;

use App\Models\Accounting;

class ReceiptPaymentExport implements FromCollection, WithHeadings, WithTitle, WithStyles, WithEvents
{
    protected $startDate;
    protected $endDate;
    protected $accountId;
    protected $rTotal;
    protected $pTotal;
    
    // Accept start and end dates in the constructor
    public function __construct($startDate, $endDate, $accountId)
    {
        $this->startDate = $startDate;
        $this->endDate = $endDate;
        $this->accountId = $accountId;
        $this->rTotal = 0;
        $this->pTotal = 0;
    }
    
    /**
    * @return \Illuminate\Support\Collection
    */
    public function collection()
    {
        $openings = Accounting::cbOpening($this->startDate,$this->accountId);
        $ledger_count = Accounting::getLedgers("all")->count();
        $voucher_amounts = Accounting::rpAccount($this->startDate,$this->endDate,$this->accountId,0,$ledger_count);
        $closings = Accounting::cbClosing($this->endDate,$this->accountId);
        
        $result = [];
        
        $receipts = $payments = [];
        foreach($openings as $opening){
            $receipts[] = ['ledger_name' => 'OPENING BALANCE : '.$opening['ledger_name'], 'amount' => $opening['amount']];
        }
        
        if(array_key_exists('receipt', $voucher_amounts)){
            foreach($voucher_amounts['receipt'] as $voucher_amount){
                $receipts[] = $voucher_amount;
            }
        }
        
        if(array_key_exists('payment', $voucher_amounts)){
            foreach($voucher_amounts['payment'] as $voucher_amount){
                $payments[] = $voucher_amount;
            }
        }
        
        foreach($closings as $closing){
            $payments[] = ['ledger_name' => 'CLOSING BALANCE : '.$closing['ledger_name'], 'amount' => $closing['amount']];
        }
        
        $total_rows = max(count($receipts), count($payments));
        
        for ($i = 0; $i < $total_rows; $i++) {
            
            // Set default empty values if there's no corresponding entry
            $receipt_ledger_name = isset($receipts[$i]) ? $receipts[$i]['ledger_name'] : '';
            $receipt_amount = isset($receipts[$i]) ? $receipts[$i]['amount'] : '';
            $payment_ledger_name = isset($payments[$i]) ? $payments[$i]['ledger_name'] : '';
            $payment_amount = isset($payments[$i]) ? $payments[$i]['amount'] : '';
            
            if($receipt_amount!=''){
                $this->rTotal += $receipt_amount;
            }
            
            if($payment_amount!=''){
                $this->pTotal += $payment_amount;
            }
                
            // Add the row to the result array
            $result[] = [
                $receipt_ledger_name,  // Column A: Ledger Name (Receipt)
                $receipt_amount,       // Column B: Total Amount (Receipt)
                $payment_ledger_name,   // Column C: Ledger Name (Payment)
                $payment_amount         // Column D: Total Amount (Payment)
            ];
        }
        
        return collect($result);
    }
    
    public function headings(): array
    {
        return [
            'Receipts',
            'Amount',
            'Payments',
            'Amount',
        ];
    }
    
    public function title(): string
    {
        return 'Receipts & Payments';
    }
    
    public function styles(Worksheet $sheet)
    {
        $sheet->getColumnDimension('A')->setWidth(50);
        $sheet->getColumnDimension('B')->setWidth(20);
        $sheet->getColumnDimension('C')->setWidth(50);
        $sheet->getColumnDimension('D')->setWidth(20);
            
        $sheet->getStyle('A1:D1')->applyFromArray([
            'font' => [
                'bold' => true,
                'size' => 12,
            ],
            'alignment' => [
                'horizontal' => \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER,
                'vertical' => \PhpOffice\PhpSpreadsheet\Style\Alignment::VERTICAL_CENTER,
            ],
            'fill' => [
                'fillType' => \PhpOffice\PhpSpreadsheet\Style\Fill::FILL_SOLID,
                'startColor' => [
                    'rgb' => 'FFFF00',
                ]
            ]
        ]);
        
        return [];
    }
    
    public function registerEvents(): array
    {
        return [
            AfterSheet::class => function (AfterSheet $event) {
                $sheet = $event->sheet;
                    
                // Insert the title row before the column headings (row 1)
                $sheet->insertNewRowBefore(1, 1); // Insert a new row at position 1 for the title
                $sheet->setCellValue('A1', 'Receipts & Payments From '.date('d-m-Y',strtotime($this->startDate)).' To '.date('d-m-Y',strtotime($this->endDate)));
                $sheet->mergeCells('A1:D1');
                $sheet->getStyle('A1')->applyFromArray([
                    'font' => [
                        'bold' => true,
                        'size' => 14,
                    ],
                    'alignment' => [
                        'horizontal' => \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_CENTER,
                    ]
                ]);
                
                $lastRow = $sheet->getHighestRow()+1;
                $sheet->setCellValue('A'.$lastRow, 'TOTAL');
                $sheet->setCellValue('B'.$lastRow, $this->rTotal);
                $sheet->setCellValue('C'.$lastRow, 'TOTAL');
                $sheet->setCellValue('D'.$lastRow, $this->pTotal);
                
                $sheet->getStyle('A'.$lastRow.':D'.$lastRow)->applyFromArray([
                    'font' => [
                        'bold' => true,
                    ]
                ]);
                
                $sheet->getStyle('A2:D'.$lastRow)->applyFromArray([
                    'font' => [
                        'name' => 'Calibri',
                        'size' => 12,
                    ],
                    'borders' => [
                        'allBorders' => [
                            'borderStyle' => \PhpOffice\PhpSpreadsheet\Style\Border::BORDER_THIN,
                            'color' => ['argb' => '000000'],
                        ],
                    ],
                ]);
                    
                $sheet->getRowDimension(1)->setRowHeight(25);
                $sheet->getRowDimension(2)->setRowHeight(20);
                
                $sheet->getStyle('B3:B'.$lastRow)->applyFromArray([
                    'numberFormat' => [
                        'formatCode' => '0.00',
                    ],
                    'alignment' => [
                        'horizontal' => \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_RIGHT,
                    ]
                ]);
                
                $sheet->getStyle('D3:D'.$lastRow)->applyFromArray([
                    'numberFormat' => [
                        'formatCode' => '0.00',
                    ],
                    'alignment' => [
                        'horizontal' => \PhpOffice\PhpSpreadsheet\Style\Alignment::HORIZONTAL_RIGHT,
                    ]
                ]);
            }
        ];
    }
}
