<?php

namespace App\Http\Controllers\Accounts;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Validator;

use App\Models\Financialyear;
use App\Models\Accounttype;
use App\Models\Vouchertype;
use App\Models\Voucher;
use App\Models\Ledger;
use App\Models\MenuAccess;

class Vouchers extends Controller
{
    protected $accountId;
    protected $fyearId;
    protected $fsDate;
    protected $feDate;

    public function __construct()
    {
        $this->accountId = Accounttype::getDefaultAccountId();
        $this->fyearId = Financialyear::getDefaultFyearId($this->accountId);
        $fyear = Financialyear::find($this->fyearId);
        if ($fyear) {
            $this->fsDate = $fyear->sdate;
            $this->feDate = $fyear->edate;
        } else {
            // Handle missing Financial Year data gracefully
            abort(404, 'Financial Year not found');
        }
    }
    
    /** Payment Voucher **/

    public function payment()
    {
        try {
            // Validate and sanitize input values if applicable (assuming $accountId and $fyearId are user-controlled)
            $accountId = filter_var($this->accountId, FILTER_VALIDATE_INT); // Example integer validation
            $fyearId = filter_var($this->fyearId, FILTER_VALIDATE_INT);
            $fdate = $this->fsDate;
            $tdate = now()->format('Y-m-d'); // Improved date handling
                
            $vtypes = cache()->remember('vouchertypes_vid1', 60, function () {
                return Vouchertype::where('vid', 1)->orderBy('id', 'ASC')->get();
            });
                
            // Return view with sanitized and validated data
            return view('account.voucher.payment.report', compact('accountId', 'fyearId', 'fdate', 'tdate', 'vtypes'));
                
        } catch (\Exception $e) {
            // Log any errors for debugging
            Log::error('Payment Report Generation Error: ' . $e->getMessage());
                
            // Provide a user-friendly error response
            return redirect()->route('home')
                             ->with('message', 'An error occurred while generating the payment report.')
                             ->with('alert-type', 'error');
        }
    }
    
    public function payment_ajax(Request $request)
    {
        // Define allowed columns for ordering to prevent SQL injection
        $allowedOrderColumns = [null, 'vno', 'vdate','vtype_name', null, 'vamount'];
        $orderColumnIndex = $request->input('order.0.column');
        $order = isset($allowedOrderColumns[$orderColumnIndex]) ? $allowedOrderColumns[$orderColumnIndex] : 'vouchers.id';
        $dir = $request->input('order.0.dir') === 'asc' ? 'asc' : 'desc';
            
        // Use query builder for better performance and scalability
        $query = Voucher::join('vouchertypes','vouchers.vtype_id','=','vouchertypes.id')
                        ->where('vouchertypes.vid', 1)
                        ->where('vouchers.account_id', $this->accountId);
            
        $fdate = $request->get('fdate') ?: $this->fsDate;
        $tdate = $request->get('tdate') ?: $this->feDate;
            
        // Input sanitization for dates
        $fdate = $this->validateDate($fdate) ? $fdate : $this->fsDate;
        $tdate = $this->validateDate($tdate) ? $tdate : $this->feDate;
            
        $query->whereBetween('vdate', [$fdate, $tdate]);
            
        if ($vno = $request->get('vno')) {
            $query->where('vno', $vno);
        }
        
        if ($vtype = $request->get('vtype')) {
            $query->where('vtype_id', $vtype);
        }
            
        if ($search = $request->input('search.value')) {
            $query->where(function ($qry) use ($search) {
                $qry->where('vno', 'like', '%' . $search . '%')
                    ->orWhere('vdate', 'like', $search . '%')
                    ->orWhere('vamount', 'like', '%' . $search . '%');
            });
        }
            
        $total_data = $query->count();
        $total_filtered = $total_data;
            
        // Apply pagination
        $vouchers = $query->select('vouchers.*','vouchertypes.vtype_name')
                        ->orderBy($order, $dir)
                        ->offset($request->input('start'))
                        ->limit($request->input('length'))
                        ->get();
            
        $edit = MenuAccess::have_access(587) ? 1 : 0;
        $delete = MenuAccess::have_access(588) ? 1 : 0;
            
        $data = $vouchers->map(function ($voucher, $index) use ($edit, $delete, $request) {
            $row = [
                'sno' => $request->input('start') + $index + 1,
                'vno' => $voucher->vno,
                'date' => $voucher->vdate_formatted,
                'vtype' => $voucher->vtype_name,
                'particular' => optional($voucher->debit)->ledger_name . ' To ' . optional($voucher->credit)->ledger_name,
                'amount' => $voucher->vamount,
                'action' => $this->generateActionLinks($voucher->id, $edit, $delete, 'payment'),
            ];
            return $row;
        });
            
        return response()->json([
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($total_data),
            'recordsFiltered' => intval($total_filtered),
            'data' => $data,
        ]);
    }
    
    private function validateDate($date, $format = 'Y-m-d')
    {
        $d = \DateTime::createFromFormat($format, $date);
        return $d && $d->format($format) === $date;
    }

    private function generateActionLinks($id, $edit, $delete, $routePrefix)
    {
        // Ensure id is sanitized before encrypting
        $encryptedId = encrypt($id);
        $html = '';
    
        // Generate Edit Link if applicable
        if ($edit) {
            $editUrl = route("accounts.$routePrefix.voucher.edit", $encryptedId);
            $html .= '<a href="' . htmlspecialchars($editUrl, ENT_QUOTES, 'UTF-8') . '" title="Edit">'
                    . '<i class="icon-sm icon-pencil text-success"></i></a>&nbsp;&nbsp;&nbsp;';
        }
    
        // Generate Delete Link if applicable
        if ($delete) {
            $deleteUrl = route("accounts.$routePrefix.voucher.delete", $encryptedId);
            $html .= '<a href="' . htmlspecialchars($deleteUrl, ENT_QUOTES, 'UTF-8') . '" title="Delete" '
                    . 'onClick="return confirm(\'Are you sure you want to delete?\');">'
                    . '<i class="icon-sm icon-trash text-danger"></i></a>';
        }
    
        return $html;
    }
        
    public function payment_add()
    {
        $vtypes = cache()->remember('vouchertypes_vid1', 60, function () {
                return Vouchertype::where('vid', 1)->orderBy('id', 'ASC')->get();
            });
        $vno = Voucher::where('vtype_id', 1)
                  ->where('account_id', $this->accountId)
                  ->whereBetween('vdate', [$this->fsDate, $this->feDate])
                  ->max('vno');

        // Fallback to 1 if no existing voucher found
        $vno = is_null($vno) ? 1 : $vno + 1;
                      
        $ledgers = cache()->remember('ledgers_list', 60, function () {
            return Ledger::all(['ledger_id', 'ledger_name', 'ag_id']);
        });
            
        return view('account.voucher.payment.add', compact('vtypes', 'vno', 'ledgers'));
    }
        
    public function payment_save(Request $request)
    {
        $validated = $request->validate([
            'voucher_type' => 'required|integer|exists:vouchertypes,id',
            'voucher_no' => 'required|integer',
            'voucher_date' => "required|date|after_or_equal:$this->fsDate|before_or_equal:$this->feDate",
            'debit' => 'required|integer|exists:ledgers,ledger_id',
            'credit' => 'required|integer|exists:ledgers,ledger_id',
            'amount' => 'required|numeric|min:0.01',
            'cdno' => 'nullable',
            'cto' => 'nullable',
            'cdate' => 'nullable|date',
            'remarks' => 'nullable',
        ]);
            
        DB::beginTransaction();
        try {
            $group_id = Voucher::withTrashed()->max('group_id') + 1 ?? 1;
            $vno = Voucher::where('vtype_id', $request->voucher_type)
                          ->where('account_id', $this->accountId)
                          ->whereBetween('vdate', [$this->fsDate, $this->feDate])
                          ->max('vno') + 1 ?? 1;
                
            // Create voucher entry
            $voucher = Voucher::create([
                'group_id' => $group_id,
                'vtype_id' => $validated['voucher_type'],
                'vno' => $vno,
                'vorder' => 1,
                'vdate' => $validated['voucher_date'],
                'dr_ledger' => $validated['debit'],
                'cr_ledger' => $validated['credit'],
                'vamount' => $validated['amount'],
                'cdno' => $validated['cdno'],
                'cto'=> $validated['cto'],
                'cdate' => $validated['cdate'],
                'remarks' => $validated['remarks'],
                'account_id' => $this->accountId,
                'status' => 1,
                'created_by' => auth()->id(),
            ]);
                
            DB::commit();
                
            return redirect()->route('accounts.payment.voucher')
                            ->with('message', 'Payment Voucher Created Successfully!')
                            ->with('alert-type', 'success');
        } catch (\Exception $e) {
            DB::rollBack();
            // Log error for debugging
            Log::error('Payment Save Error: ' . $e->getMessage());
            // Failure message
            return redirect()->route('accounts.payment.voucher')
                            ->with('message', 'Payment Voucher Could not be Created!')
                            ->with('alert-type', 'error');
        }
    }
        
    public function payment_edit($vid)
    {
        try {
            // Decrypt the voucher ID and handle potential decryption issues
            $voucher_id = decrypt($vid);
            $voucher = Voucher::findOrFail($voucher_id); // Use findOrFail for better error handling
            
            // Optimize query by selecting only necessary columns (if applicable)
            $vtypes = cache()->remember('vouchertypes_vid1', 60, function () {
                return Vouchertype::where('vid', 1)->orderBy('id', 'ASC')->get();
            });
            $ledgers = cache()->remember('ledgers_list', 60, function () {
                return Ledger::all(['ledger_id', 'ledger_name', 'ag_id']);
            });
            
            return view('account.voucher.payment.edit', compact('vtypes', 'ledgers', 'voucher'));
        } catch (\Illuminate\Contracts\Encryption\DecryptException $e) {
            return redirect()->route('accounts.payment.voucher')
                            ->with('message', 'Invalid Voucher!')
                            ->with('alert-type', 'error');
        } catch (\Exception $e) {
            Log::error('Voucher Edit Error: ' . $e->getMessage());
            return redirect()->route('accounts.payment.voucher')
                            ->with('message', 'An error occurred!')
                            ->with('alert-type', 'error');
        }
    }
        
    public function payment_update(Request $request)
    {
        $validated = $request->validate([
            'voucher_id' => 'required|string', // Ensure voucher_id is present
            'voucher_date' => "required|date|after_or_equal:$this->fsDate|before_or_equal:$this->feDate",
            'debit' => 'required|integer|exists:ledgers,ledger_id',
            'credit' => 'required|integer|exists:ledgers,ledger_id',
            'amount' => 'required|numeric|min:0.01',
            'cdno' => 'nullable',
            'cto' => 'nullable',
            'cdate' => 'nullable|date',
            'remarks' => 'nullable',
        ]);
            
        try {
            // Decrypt and fetch voucher securely
            $voucher_id = decrypt($validated['voucher_id']);
            $voucher = Voucher::findOrFail($voucher_id);
                
            DB::beginTransaction();
                
            $voucher->update([
                'vdate' => $validated['voucher_date'],
                'dr_ledger' => $validated['debit'],
                'cr_ledger' => $validated['credit'],
                'vamount' => $validated['amount'],
                'cdno' => $validated['cdno'],
                'cto'=> $validated['cto'],
                'cdate' => $validated['cdate'],
                'remarks' => $validated['remarks'],
            ]);
                
            DB::commit();
                
            return redirect()->route('accounts.payment.voucher')
                        ->with('message', 'Payment Voucher Updated Successfully!')
                        ->with('alert-type', 'success');
        } catch (\Illuminate\Contracts\Encryption\DecryptException $e) {
            return redirect()->route('accounts.payment.voucher')
                        ->with('message', 'Invalid Voucher!')
                        ->with('alert-type', 'error');
        } catch (\Exception $e) {
            DB::rollBack();
            Log::error('Payment Save Error: ' . $e->getMessage());
            return redirect()->route('accounts.payment.voucher')
                            ->with('message', 'Payment Voucher Could not be Updated!')
                            ->with('alert-type', 'error');
        }
    }
        
    public function payment_delete($vid)
    {
        try {
            // Decrypt the voucher ID and handle potential decryption issues
            $voucher_id = decrypt($vid);
            $voucher = Voucher::findOrFail($voucher_id); // Use findOrFail for better error handling
            
            $voucher->delete();
            
            return redirect()->route('accounts.payment.voucher')
                        ->with('message', 'Payment Voucher Deleted Successfully!')
                        ->with('alert-type', 'success');
        } catch (\Illuminate\Contracts\Encryption\DecryptException $e) {
            return redirect()->route('accounts.payment.voucher')
                            ->with('message', 'Invalid Voucher!')
                            ->with('alert-type', 'error');
        } catch (\Exception $e) {
            Log::error('Voucher Edit Error: ' . $e->getMessage());
            return redirect()->route('accounts.payment.voucher')
                            ->with('message', 'An error occurred!')
                            ->with('alert-type', 'error');
        }
    }
    
    /** Receipt Voucher **/
        
    public function receipt(){
        try {
            // Validate and sanitize input values if applicable (assuming $accountId and $fyearId are user-controlled)
            $accountId = filter_var($this->accountId, FILTER_VALIDATE_INT); // Example integer validation
            $fyearId = filter_var($this->fyearId, FILTER_VALIDATE_INT);
            $fdate = $this->fsDate;
            $tdate = now()->format('Y-m-d'); // Improved date handling
                
            $vtypes = cache()->remember('vouchertypes_vid2', 60, function () {
                return Vouchertype::where('vid', 2)->orderBy('id', 'ASC')->get();
            });
                
            // Return view with sanitized and validated data
            return view('account.voucher.receipt.report', compact('accountId', 'fyearId', 'fdate', 'tdate', 'vtypes'));
                
        } catch (\Exception $e) {
            // Log any errors for debugging
            Log::error('Receipt Report Generation Error: ' . $e->getMessage());
                
            // Provide a user-friendly error response
            return redirect()->route('home')
                             ->with('message', 'An error occurred while generating the receipt report.')
                             ->with('alert-type', 'error');
        }
    }
        
    public function receipt_ajax(Request $request)
    {
        // Define allowed columns for ordering to prevent SQL injection
        $allowedOrderColumns = [null, 'vno', 'vdate','vtype_name', null, 'vamount'];
        $orderColumnIndex = $request->input('order.0.column');
        $order = isset($allowedOrderColumns[$orderColumnIndex]) ? $allowedOrderColumns[$orderColumnIndex] : 'vouchers.id';
        $dir = $request->input('order.0.dir') === 'asc' ? 'asc' : 'desc';
            
        // Use query builder for better performance and scalability
        $query = Voucher::join('vouchertypes','vouchers.vtype_id','=','vouchertypes.id')
                        ->where('vouchertypes.vid', 2)
                        ->where('vouchers.account_id', $this->accountId);
            
        $fdate = $request->get('fdate') ?: $this->fsDate;
        $tdate = $request->get('tdate') ?: $this->feDate;
            
        // Input sanitization for dates
        $fdate = $this->validateDate($fdate) ? $fdate : $this->fsDate;
        $tdate = $this->validateDate($tdate) ? $tdate : $this->feDate;
            
        $query->whereBetween('vdate', [$fdate, $tdate]);
            
        if ($vno = $request->get('vno')) {
            $query->where('vno', $vno);
        }
        
        if ($vtype = $request->get('vtype')) {
            $query->where('vtype_id', $vtype);
        }
            
        if ($search = $request->input('search.value')) {
            $query->where(function ($qry) use ($search) {
                $qry->where('vno', 'like', '%' . $search . '%')
                    ->orWhere('vdate', 'like', $search . '%')
                    ->orWhere('vamount', 'like', '%' . $search . '%');
            });
        }
            
        $total_data = $query->count();
        $total_filtered = $total_data;
            
        // Apply pagination
        $vouchers = $query->select('vouchers.*','vouchertypes.vtype_name')
                        ->orderBy($order, $dir)
                        ->offset($request->input('start'))
                        ->limit($request->input('length'))
                        ->get();
            
        $edit = MenuAccess::have_access(596) ? 1 : 0;
        $delete = MenuAccess::have_access(597) ? 1 : 0;
            
        $data = $vouchers->map(function ($voucher, $index) use ($edit, $delete, $request) {
            $row = [
                'sno' => $request->input('start') + $index + 1,
                'vno' => $voucher->vno,
                'date' => $voucher->vdate_formatted,
                'vtype' => $voucher->vtype_name,
                'particular' => optional($voucher->debit)->ledger_name . ' To ' . optional($voucher->credit)->ledger_name,
                'amount' => $voucher->vamount,
                'action' => $this->generateActionLinks($voucher->id, $edit, $delete, 'receipt'),
            ];
            return $row;
        });
            
        return response()->json([
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($total_data),
            'recordsFiltered' => intval($total_filtered),
            'data' => $data,
        ]);
    }
        
    public function receipt_add()
    {
        $vtypes = cache()->remember('vouchertypes_vid2', 60, function () {
                return Vouchertype::where('vid', 2)->orderBy('id', 'ASC')->get();
            });
        $vno = Voucher::where('vtype_id', 2)
                  ->where('account_id', $this->accountId)
                  ->whereBetween('vdate', [$this->fsDate, $this->feDate])
                  ->max('vno');
            
        // Fallback to 1 if no existing voucher found
        $vno = is_null($vno) ? 1 : $vno + 1;
                      
        $ledgers = cache()->remember('ledgers_list', 60, function () {
                return Ledger::all(['ledger_id', 'ledger_name', 'ag_id']);
            });
            
        return view('account.voucher.receipt.add', compact('vtypes', 'vno', 'ledgers'));
    }
    
    public function receipt_save(Request $request)
    {
        $validated = $request->validate([
            'voucher_type' => 'required|integer|exists:vouchertypes,id',
            'voucher_no' => 'required|integer',
            'voucher_date' => "required|date|after_or_equal:$this->fsDate|before_or_equal:$this->feDate",
            'debit' => 'required|integer|exists:ledgers,ledger_id',
            'credit' => 'required|integer|exists:ledgers,ledger_id',
            'amount' => 'required|numeric|min:0.01',
            'cdno' => 'nullable',
            'cto' => 'nullable',
            'cdate' => 'nullable|date',
            'remarks' => 'nullable',
        ]);
            
        DB::beginTransaction();
        try {
            $group_id = Voucher::withTrashed()->max('group_id') + 1 ?? 1;
            $vno = Voucher::where('vtype_id', $request->voucher_type)
                          ->where('account_id', $this->accountId)
                          ->whereBetween('vdate', [$this->fsDate, $this->feDate])
                          ->max('vno') + 1 ?? 1;
                
            // Create voucher entry
            $voucher = Voucher::create([
                'group_id' => $group_id,
                'vtype_id' => $validated['voucher_type'],
                'vno' => $vno,
                'vorder' => 1,
                'vdate' => $validated['voucher_date'],
                'dr_ledger' => $validated['debit'],
                'cr_ledger' => $validated['credit'],
                'vamount' => $validated['amount'],
                'cdno' => $validated['cdno'],
                'cto'=> $validated['cto'],
                'cdate' => $validated['cdate'],
                'remarks' => $validated['remarks'],
                'account_id' => $this->accountId,
                'status' => 1,
                'created_by' => auth()->id(),
            ]);
                
            DB::commit();
                
            return redirect()->route('accounts.receipt.voucher')
                            ->with('message', 'Receipt Voucher Created Successfully!')
                            ->with('alert-type', 'success');
        } catch (\Exception $e) {
            DB::rollBack();
            // Log error for debugging
            Log::error('Receipt Save Error: ' . $e->getMessage());
            // Failure message
            return redirect()->route('accounts.receipt.voucher')
                            ->with('message', 'Receipt Voucher Could not be Created!')
                            ->with('alert-type', 'error');
        }
    }
    
    public function receipt_edit($vid)
    {
        try {
            // Decrypt the voucher ID and handle potential decryption issues
            $voucher_id = decrypt($vid);
            $voucher = Voucher::findOrFail($voucher_id); // Use findOrFail for better error handling
            
            // Optimize query by selecting only necessary columns (if applicable)
            $vtypes = cache()->remember('vouchertypes_vid2', 60, function () {
                return Vouchertype::where('vid', 2)->orderBy('id', 'ASC')->get();
            });
            $ledgers = cache()->remember('ledgers_list', 60, function () {
                return Ledger::all(['ledger_id', 'ledger_name', 'ag_id']);
            });
            
            return view('account.voucher.receipt.edit', compact('vtypes', 'ledgers', 'voucher'));
        } catch (\Illuminate\Contracts\Encryption\DecryptException $e) {
            return redirect()->route('accounts.receipt.voucher')
                            ->with('message', 'Invalid Voucher!')
                            ->with('alert-type', 'error');
        } catch (\Exception $e) {
            Log::error('Voucher Edit Error: ' . $e->getMessage());
            return redirect()->route('accounts.receipt.voucher')
                            ->with('message', 'An error occurred!')
                            ->with('alert-type', 'error');
        }
    }
        
    public function receipt_update(Request $request)
    {
        $validated = $request->validate([
            'voucher_id' => 'required|string', // Ensure voucher_id is present
            'voucher_date' => "required|date|after_or_equal:$this->fsDate|before_or_equal:$this->feDate",
            'debit' => 'required|integer|exists:ledgers,ledger_id',
            'credit' => 'required|integer|exists:ledgers,ledger_id',
            'amount' => 'required|numeric|min:0.01',
            'cdno' => 'nullable',
            'cto' => 'nullable',
            'cdate' => 'nullable|date',
            'remarks' => 'nullable',
        ]);
            
        try {
            // Decrypt and fetch voucher securely
            $voucher_id = decrypt($validated['voucher_id']);
            $voucher = Voucher::findOrFail($voucher_id);
                
            DB::beginTransaction();
                
            $voucher->update([
                'vdate' => $validated['voucher_date'],
                'dr_ledger' => $validated['debit'],
                'cr_ledger' => $validated['credit'],
                'vamount' => $validated['amount'],
                'cdno' => $validated['cdno'],
                'cto'=> $validated['cto'],
                'cdate' => $validated['cdate'],
                'remarks' => $validated['remarks'],
            ]);
                
            DB::commit();
                
            return redirect()->route('accounts.receipt.voucher')
                        ->with('message', 'Receipt Voucher Updated Successfully!')
                        ->with('alert-type', 'success');
        } catch (\Illuminate\Contracts\Encryption\DecryptException $e) {
            return redirect()->route('accounts.receipt.voucher')
                        ->with('message', 'Invalid Voucher!')
                        ->with('alert-type', 'error');
        } catch (\Exception $e) {
            DB::rollBack();
            Log::error('Receipt Save Error: ' . $e->getMessage());
            return redirect()->route('accounts.receipt.voucher')
                            ->with('message', 'Receipt Voucher Could not be Updated!')
                            ->with('alert-type', 'error');
        }
    }
        
    public function receipt_delete($vid)
    {
        try {
            // Decrypt the voucher ID and handle potential decryption issues
            $voucher_id = decrypt($vid);
            $voucher = Voucher::findOrFail($voucher_id); // Use findOrFail for better error handling
            
            $voucher->delete();
            
            return redirect()->route('accounts.receipt.voucher')
                        ->with('message', 'Receipt Voucher Deleted Successfully!')
                        ->with('alert-type', 'success');
        } catch (\Illuminate\Contracts\Encryption\DecryptException $e) {
            return redirect()->route('accounts.receipt.voucher')
                            ->with('message', 'Invalid Voucher!')
                            ->with('alert-type', 'error');
        } catch (\Exception $e) {
            Log::error('Voucher Edit Error: ' . $e->getMessage());
            return redirect()->route('accounts.receipt.voucher')
                            ->with('message', 'An error occurred!')
                            ->with('alert-type', 'error');
        }
    }
    
    /** Contra Voucher **/
        
    public function contra(){
        try {
            // Validate and sanitize input values if applicable (assuming $accountId and $fyearId are user-controlled)
            $accountId = filter_var($this->accountId, FILTER_VALIDATE_INT); // Example integer validation
            $fyearId = filter_var($this->fyearId, FILTER_VALIDATE_INT);
            $fdate = $this->fsDate;
            $tdate = now()->format('Y-m-d'); // Improved date handling
                
            $vtypes = cache()->remember('vouchertypes_vid3', 60, function () {
                return Vouchertype::where('vid', 3)->orderBy('id', 'ASC')->get();
            });
                
            // Return view with sanitized and validated data
            return view('account.voucher.contra.report', compact('accountId', 'fyearId', 'fdate', 'tdate', 'vtypes'));
                
        } catch (\Exception $e) {
            // Log any errors for debugging
            Log::error('Contra Report Generation Error: ' . $e->getMessage());
                
            // Provide a user-friendly error response
            return redirect()->route('home')
                             ->with('message', 'An error occurred while generating the contra report.')
                             ->with('alert-type', 'error');
        }
    }
        
    public function contra_ajax(Request $request)
    {
        // Define allowed columns for ordering to prevent SQL injection
        $allowedOrderColumns = [null, 'vno', 'vdate','vtype_name', null, 'vamount'];
        $orderColumnIndex = $request->input('order.0.column');
        $order = isset($allowedOrderColumns[$orderColumnIndex]) ? $allowedOrderColumns[$orderColumnIndex] : 'vouchers.id';
        $dir = $request->input('order.0.dir') === 'asc' ? 'asc' : 'desc';
            
        // Use query builder for better performance and scalability
        $query = Voucher::join('vouchertypes','vouchers.vtype_id','=','vouchertypes.id')
                        ->where('vouchertypes.vid', 3)
                        ->where('vouchers.account_id', $this->accountId);
            
        $fdate = $request->get('fdate') ?: $this->fsDate;
        $tdate = $request->get('tdate') ?: $this->feDate;
            
        // Input sanitization for dates
        $fdate = $this->validateDate($fdate) ? $fdate : $this->fsDate;
        $tdate = $this->validateDate($tdate) ? $tdate : $this->feDate;
            
        $query->whereBetween('vdate', [$fdate, $tdate]);
            
        if ($vno = $request->get('vno')) {
            $query->where('vno', $vno);
        }
        
        if ($vtype = $request->get('vtype')) {
            $query->where('vtype_id', $vtype);
        }
            
        if ($search = $request->input('search.value')) {
            $query->where(function ($qry) use ($search) {
                $qry->where('vno', 'like', '%' . $search . '%')
                    ->orWhere('vdate', 'like', $search . '%')
                    ->orWhere('vamount', 'like', '%' . $search . '%');
            });
        }
            
        $total_data = $query->count();
        $total_filtered = $total_data;
            
        // Apply pagination
        $vouchers = $query->select('vouchers.*','vouchertypes.vtype_name')
                        ->orderBy($order, $dir)
                        ->offset($request->input('start'))
                        ->limit($request->input('length'))
                        ->get();
            
        $edit = MenuAccess::have_access(599) ? 1 : 0;
        $delete = MenuAccess::have_access(600) ? 1 : 0;
            
        $data = $vouchers->map(function ($voucher, $index) use ($edit, $delete, $request) {
            $row = [
                'sno' => $request->input('start') + $index + 1,
                'vno' => $voucher->vno,
                'date' => $voucher->vdate_formatted,
                'vtype' => $voucher->vtype_name,
                'particular' => optional($voucher->debit)->ledger_name . ' To ' . optional($voucher->credit)->ledger_name,
                'amount' => $voucher->vamount,
                'action' => $this->generateActionLinks($voucher->id, $edit, $delete, 'contra'),
            ];
            return $row;
        });
            
        return response()->json([
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($total_data),
            'recordsFiltered' => intval($total_filtered),
            'data' => $data,
        ]);
    }
        
    public function contra_add()
    {
        $vtypes = cache()->remember('vouchertypes_vid3', 60, function () {
                return Vouchertype::where('vid', 3)->orderBy('id', 'ASC')->get();
            });
        $vno = Voucher::where('vtype_id', 3)
                  ->where('account_id', $this->accountId)
                  ->whereBetween('vdate', [$this->fsDate, $this->feDate])
                  ->max('vno');
            
        // Fallback to 1 if no existing voucher found
        $vno = is_null($vno) ? 1 : $vno + 1;
                      
        $ledgers = cache()->remember('ledgers_list', 60, function () {
                return Ledger::all(['ledger_id', 'ledger_name', 'ag_id']);
            });
            
        return view('account.voucher.contra.add', compact('vtypes', 'vno', 'ledgers'));
    }
    
    public function contra_save(Request $request)
    {
        $validated = $request->validate([
            'voucher_type' => 'required|integer|exists:vouchertypes,id',
            'voucher_no' => 'required|integer',
            'voucher_date' => "required|date|after_or_equal:$this->fsDate|before_or_equal:$this->feDate",
            'debit' => 'required|integer|exists:ledgers,ledger_id',
            'credit' => 'required|integer|exists:ledgers,ledger_id',
            'amount' => 'required|numeric|min:0.01',
            'cdno' => 'nullable',
            'cto' => 'nullable',
            'cdate' => 'nullable|date',
            'remarks' => 'nullable',
        ]);
            
        DB::beginTransaction();
        try {
            $group_id = Voucher::withTrashed()->max('group_id') + 1 ?? 1;
            $vno = Voucher::where('vtype_id', $request->voucher_type)
                          ->where('account_id', $this->accountId)
                          ->whereBetween('vdate', [$this->fsDate, $this->feDate])
                          ->max('vno') + 1 ?? 1;
                
            // Create voucher entry
            $voucher = Voucher::create([
                'group_id' => $group_id,
                'vtype_id' => $validated['voucher_type'],
                'vno' => $vno,
                'vorder' => 1,
                'vdate' => $validated['voucher_date'],
                'dr_ledger' => $validated['debit'],
                'cr_ledger' => $validated['credit'],
                'vamount' => $validated['amount'],
                'cdno' => $validated['cdno'],
                'cto'=> $validated['cto'],
                'cdate' => $validated['cdate'],
                'remarks' => $validated['remarks'],
                'account_id' => $this->accountId,
                'status' => 1,
                'created_by' => auth()->id(),
            ]);
                
            DB::commit();
                
            return redirect()->route('accounts.contra.voucher')
                            ->with('message', 'Contra Voucher Created Successfully!')
                            ->with('alert-type', 'success');
        } catch (\Exception $e) {
            DB::rollBack();
            // Log error for debugging
            Log::error('Contra Save Error: ' . $e->getMessage());
            // Failure message
            return redirect()->route('accounts.contra.voucher')
                            ->with('message', 'Contra Voucher Could not be Created!')
                            ->with('alert-type', 'error');
        }
    }
    
    public function contra_edit($vid)
    {
        try {
            // Decrypt the voucher ID and handle potential decryption issues
            $voucher_id = decrypt($vid);
            $voucher = Voucher::findOrFail($voucher_id); // Use findOrFail for better error handling
            
            // Optimize query by selecting only necessary columns (if applicable)
            $vtypes = cache()->remember('vouchertypes_vid3', 60, function () {
                return Vouchertype::where('vid', 3)->orderBy('id', 'ASC')->get();
            });
            $ledgers = cache()->remember('ledgers_list', 60, function () {
                return Ledger::all(['ledger_id', 'ledger_name', 'ag_id']);
            });
            
            return view('account.voucher.contra.edit', compact('vtypes', 'ledgers', 'voucher'));
        } catch (\Illuminate\Contracts\Encryption\DecryptException $e) {
            return redirect()->route('accounts.contra.voucher')
                            ->with('message', 'Invalid Voucher!')
                            ->with('alert-type', 'error');
        } catch (\Exception $e) {
            Log::error('Voucher Edit Error: ' . $e->getMessage());
            return redirect()->route('accounts.contra.voucher')
                            ->with('message', 'An error occurred!')
                            ->with('alert-type', 'error');
        }
    }
        
    public function contra_update(Request $request)
    {
        $validated = $request->validate([
            'voucher_id' => 'required|string', // Ensure voucher_id is present
            'voucher_date' => "required|date|after_or_equal:$this->fsDate|before_or_equal:$this->feDate",
            'debit' => 'required|integer|exists:ledgers,ledger_id',
            'credit' => 'required|integer|exists:ledgers,ledger_id',
            'amount' => 'required|numeric|min:0.01',
            'cdno' => 'nullable',
            'cto' => 'nullable',
            'cdate' => 'nullable|date',
            'remarks' => 'nullable',
        ]);
            
        try {
            // Decrypt and fetch voucher securely
            $voucher_id = decrypt($validated['voucher_id']);
            $voucher = Voucher::findOrFail($voucher_id);
                
            DB::beginTransaction();
                
            $voucher->update([
                'vdate' => $validated['voucher_date'],
                'dr_ledger' => $validated['debit'],
                'cr_ledger' => $validated['credit'],
                'vamount' => $validated['amount'],
                'cdno' => $validated['cdno'],
                'cto'=> $validated['cto'],
                'cdate' => $validated['cdate'],
                'remarks' => $validated['remarks'],
            ]);
                
            DB::commit();
                
            return redirect()->route('accounts.contra.voucher')
                        ->with('message', 'Contra Voucher Updated Successfully!')
                        ->with('alert-type', 'success');
        } catch (\Illuminate\Contracts\Encryption\DecryptException $e) {
            return redirect()->route('accounts.contra.voucher')
                        ->with('message', 'Invalid Voucher!')
                        ->with('alert-type', 'error');
        } catch (\Exception $e) {
            DB::rollBack();
            Log::error('Contra Save Error: ' . $e->getMessage());
            return redirect()->route('accounts.contra.voucher')
                            ->with('message', 'Contra Voucher Could not be Updated!')
                            ->with('alert-type', 'error');
        }
    }
        
    public function contra_delete($vid)
    {
        try {
            // Decrypt the voucher ID and handle potential decryption issues
            $voucher_id = decrypt($vid);
            $voucher = Voucher::findOrFail($voucher_id); // Use findOrFail for better error handling
            
            $voucher->delete();
            
            return redirect()->route('accounts.contra.voucher')
                        ->with('message', 'Contra Voucher Deleted Successfully!')
                        ->with('alert-type', 'success');
        } catch (\Illuminate\Contracts\Encryption\DecryptException $e) {
            return redirect()->route('accounts.contra.voucher')
                            ->with('message', 'Invalid Voucher!')
                            ->with('alert-type', 'error');
        } catch (\Exception $e) {
            Log::error('Voucher Edit Error: ' . $e->getMessage());
            return redirect()->route('accounts.contra.voucher')
                            ->with('message', 'An error occurred!')
                            ->with('alert-type', 'error');
        }
    }
    
    /** Journal Voucher **/
        
    public function journal(){
        try {
            // Validate and sanitize input values if applicable (assuming $accountId and $fyearId are user-controlled)
            $accountId = filter_var($this->accountId, FILTER_VALIDATE_INT); // Example integer validation
            $fyearId = filter_var($this->fyearId, FILTER_VALIDATE_INT);
            $fdate = $this->fsDate;
            $tdate = now()->format('Y-m-d'); // Improved date handling
                
            $vtypes = cache()->remember('vouchertypes_vid4', 60, function () {
                return Vouchertype::where('vid', 4)->orderBy('id', 'ASC')->get();
            });
                
            // Return view with sanitized and validated data
            return view('account.voucher.journal.report', compact('accountId', 'fyearId', 'fdate', 'tdate', 'vtypes'));
                
        } catch (\Exception $e) {
            // Log any errors for debugging
            Log::error('Journal Report Generation Error: ' . $e->getMessage());
                
            // Provide a user-friendly error response
            return redirect()->route('home')
                             ->with('message', 'An error occurred while generating the journal report.')
                             ->with('alert-type', 'error');
        }
    }
        
    public function journal_ajax(Request $request)
    {
        // Define allowed columns for ordering to prevent SQL injection
        $allowedOrderColumns = [null, 'vno', 'vdate','vtype_name', null, 'vamount'];
        $orderColumnIndex = $request->input('order.0.column');
        $order = isset($allowedOrderColumns[$orderColumnIndex]) ? $allowedOrderColumns[$orderColumnIndex] : 'vouchers.id';
        $dir = $request->input('order.0.dir') === 'asc' ? 'asc' : 'desc';
            
        // Use query builder for better performance and scalability
        $query = Voucher::join('vouchertypes','vouchers.vtype_id','=','vouchertypes.id')
                        ->where('vouchertypes.vid', 4)
                        ->where('vouchers.account_id', $this->accountId);
            
        $fdate = $request->get('fdate') ?: $this->fsDate;
        $tdate = $request->get('tdate') ?: $this->feDate;
            
        // Input sanitization for dates
        $fdate = $this->validateDate($fdate) ? $fdate : $this->fsDate;
        $tdate = $this->validateDate($tdate) ? $tdate : $this->feDate;
            
        $query->whereBetween('vdate', [$fdate, $tdate]);
            
        if ($vno = $request->get('vno')) {
            $query->where('vno', $vno);
        }
        
        if ($vtype = $request->get('vtype')) {
            $query->where('vtype_id', $vtype);
        }
            
        if ($search = $request->input('search.value')) {
            $query->where(function ($qry) use ($search) {
                $qry->where('vno', 'like', '%' . $search . '%')
                    ->orWhere('vdate', 'like', $search . '%')
                    ->orWhere('vamount', 'like', '%' . $search . '%');
            });
        }
            
        $total_data = $query->count();
        $total_filtered = $total_data;
            
        // Apply pagination
        $vouchers = $query->select('vouchers.*','vouchertypes.vtype_name')
                        ->orderBy($order, $dir)
                        ->offset($request->input('start'))
                        ->limit($request->input('length'))
                        ->get();
            
        $edit = MenuAccess::have_access(602) ? 1 : 0;
        $delete = MenuAccess::have_access(603) ? 1 : 0;
            
        $data = $vouchers->map(function ($voucher, $index) use ($edit, $delete, $request) {
            $row = [
                'sno' => $request->input('start') + $index + 1,
                'vno' => $voucher->vno,
                'date' => $voucher->vdate_formatted,
                'vtype' => $voucher->vtype_name,
                'particular' => optional($voucher->debit)->ledger_name . ' To ' . optional($voucher->credit)->ledger_name,
                'amount' => $voucher->vamount,
                'action' => $this->generateActionLinks($voucher->id, $edit, $delete, 'journal'),
            ];
            return $row;
        });
            
        return response()->json([
            'draw' => intval($request->input('draw')),
            'recordsTotal' => intval($total_data),
            'recordsFiltered' => intval($total_filtered),
            'data' => $data,
        ]);
    }
        
    public function journal_add()
    {
        $vtypes = cache()->remember('vouchertypes_vid4', 60, function () {
                return Vouchertype::where('vid', 4)->orderBy('id', 'ASC')->get();
            });
        $vno = Voucher::where('vtype_id', 4)
                  ->where('account_id', $this->accountId)
                  ->whereBetween('vdate', [$this->fsDate, $this->feDate])
                  ->max('vno');
            
        // Fallback to 1 if no existing voucher found
        $vno = is_null($vno) ? 1 : $vno + 1;
                      
        $ledgers = cache()->remember('ledgers_list', 60, function () {
                return Ledger::all(['ledger_id', 'ledger_name', 'ag_id']);
            });
            
        return view('account.voucher.journal.add', compact('vtypes', 'vno', 'ledgers'));
    }
    
    public function journal_save(Request $request)
    {
        $validated = $request->validate([
            'voucher_type' => 'required|integer|exists:vouchertypes,id',
            'voucher_no' => 'required|integer',
            'voucher_date' => "required|date|after_or_equal:$this->fsDate|before_or_equal:$this->feDate",
            'debit' => 'required|integer|exists:ledgers,ledger_id',
            'credit' => 'required|integer|exists:ledgers,ledger_id',
            'amount' => 'required|numeric|min:0.01',
            'cdno' => 'nullable',
            'cto' => 'nullable',
            'cdate' => 'nullable|date',
            'remarks' => 'nullable',
        ]);
            
        DB::beginTransaction();
        try {
            $group_id = Voucher::withTrashed()->max('group_id') + 1 ?? 1;
            $vno = Voucher::where('vtype_id', $request->voucher_type)
                          ->where('account_id', $this->accountId)
                          ->whereBetween('vdate', [$this->fsDate, $this->feDate])
                          ->max('vno') + 1 ?? 1;
                
            // Create voucher entry
            $voucher = Voucher::create([
                'group_id' => $group_id,
                'vtype_id' => $validated['voucher_type'],
                'vno' => $vno,
                'vorder' => 1,
                'vdate' => $validated['voucher_date'],
                'dr_ledger' => $validated['debit'],
                'cr_ledger' => $validated['credit'],
                'vamount' => $validated['amount'],
                'cdno' => $validated['cdno'],
                'cto'=> $validated['cto'],
                'cdate' => $validated['cdate'],
                'remarks' => $validated['remarks'],
                'account_id' => $this->accountId,
                'status' => 1,
                'created_by' => auth()->id(),
            ]);
                
            DB::commit();
                
            return redirect()->route('accounts.journal.voucher')
                            ->with('message', 'Journal Voucher Created Successfully!')
                            ->with('alert-type', 'success');
        } catch (\Exception $e) {
            DB::rollBack();
            // Log error for debugging
            Log::error('Journal Save Error: ' . $e->getMessage());
            // Failure message
            return redirect()->route('accounts.journal.voucher')
                            ->with('message', 'Journal Voucher Could not be Created!')
                            ->with('alert-type', 'error');
        }
    }
    
    public function journal_edit($vid)
    {
        try {
            // Decrypt the voucher ID and handle potential decryption issues
            $voucher_id = decrypt($vid);
            $voucher = Voucher::findOrFail($voucher_id); // Use findOrFail for better error handling
            
            // Optimize query by selecting only necessary columns (if applicable)
            $vtypes = cache()->remember('vouchertypes_vid4', 60, function () {
                return Vouchertype::where('vid', 4)->orderBy('id', 'ASC')->get();
            });
            $ledgers = cache()->remember('ledgers_list', 60, function () {
                return Ledger::all(['ledger_id', 'ledger_name', 'ag_id']);
            });
            
            return view('account.voucher.journal.edit', compact('vtypes', 'ledgers', 'voucher'));
        } catch (\Illuminate\Contracts\Encryption\DecryptException $e) {
            return redirect()->route('accounts.journal.voucher')
                            ->with('message', 'Invalid Voucher!')
                            ->with('alert-type', 'error');
        } catch (\Exception $e) {
            Log::error('Voucher Edit Error: ' . $e->getMessage());
            return redirect()->route('accounts.journal.voucher')
                            ->with('message', 'An error occurred!')
                            ->with('alert-type', 'error');
        }
    }
        
    public function journal_update(Request $request)
    {
        $validated = $request->validate([
            'voucher_id' => 'required|string', // Ensure voucher_id is present
            'voucher_date' => "required|date|after_or_equal:$this->fsDate|before_or_equal:$this->feDate",
            'debit' => 'required|integer|exists:ledgers,ledger_id',
            'credit' => 'required|integer|exists:ledgers,ledger_id',
            'amount' => 'required|numeric|min:0.01',
            'cdno' => 'nullable',
            'cto' => 'nullable',
            'cdate' => 'nullable|date',
            'remarks' => 'nullable',
        ]);
            
        try {
            // Decrypt and fetch voucher securely
            $voucher_id = decrypt($validated['voucher_id']);
            $voucher = Voucher::findOrFail($voucher_id);
                
            DB::beginTransaction();
                
            $voucher->update([
                'vdate' => $validated['voucher_date'],
                'dr_ledger' => $validated['debit'],
                'cr_ledger' => $validated['credit'],
                'vamount' => $validated['amount'],
                'cdno' => $validated['cdno'],
                'cto'=> $validated['cto'],
                'cdate' => $validated['cdate'],
                'remarks' => $validated['remarks'],
            ]);
                
            DB::commit();
                
            return redirect()->route('accounts.journal.voucher')
                        ->with('message', 'Journal Voucher Updated Successfully!')
                        ->with('alert-type', 'success');
        } catch (\Illuminate\Contracts\Encryption\DecryptException $e) {
            return redirect()->route('accounts.journal.voucher')
                        ->with('message', 'Invalid Voucher!')
                        ->with('alert-type', 'error');
        } catch (\Exception $e) {
            DB::rollBack();
            Log::error('Journal Save Error: ' . $e->getMessage());
            return redirect()->route('accounts.journal.voucher')
                            ->with('message', 'Journal Voucher Could not be Updated!')
                            ->with('alert-type', 'error');
        }
    }
        
    public function journal_delete($vid)
    {
        try {
            // Decrypt the voucher ID and handle potential decryption issues
            $voucher_id = decrypt($vid);
            $voucher = Voucher::findOrFail($voucher_id); // Use findOrFail for better error handling
            
            $voucher->delete();
            
            return redirect()->route('accounts.journal.voucher')
                        ->with('message', 'Journal Voucher Deleted Successfully!')
                        ->with('alert-type', 'success');
        } catch (\Illuminate\Contracts\Encryption\DecryptException $e) {
            return redirect()->route('accounts.journal.voucher')
                            ->with('message', 'Invalid Voucher!')
                            ->with('alert-type', 'error');
        } catch (\Exception $e) {
            Log::error('Voucher Edit Error: ' . $e->getMessage());
            return redirect()->route('accounts.journal.voucher')
                            ->with('message', 'An error occurred!')
                            ->with('alert-type', 'error');
        }
    }
}
