<?php

namespace App\Http\Controllers\Library;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\File;

use Maatwebsite\Excel\Facades\Excel;
use App\Imports\LibraryBooksImport;
use App\Exports\LibraryBooksExport;

use App\Models\Library\LibraryBook;
use App\Models\Library\LibraryCategory;
use App\Models\Library\LibrarySubcategory;
use App\Models\Library\LibraryLocation;
use App\Models\Library\LibrarySublocation;
use App\Models\MenuAccess;

use Milon\Barcode\DNS1D;
use DB;
use Image;

class Books extends Controller
{
    public function report(){
        try {
            $categories = LibraryCategory::all();
            $locations = LibraryLocation::all();
            $export = MenuAccess::have_access(670);
            $bulk = MenuAccess::have_access(665);
            
            return view('library.book.report',compact('categories','locations','export','bulk'));
        } catch (\Exception $e) {
            Log::error('Failed to fetch Books: '.$e->getMessage());
            return back()->withErrors('Failed to load Books.');
        }
    }
    
    public function getData(Request $request){
            
        $colOrder = ['book_no', 'access_no', 'title', 'author', 'subject', 'keywords', 'category', 'subcategory', 'location', 'sublocation', 'copies'];
            
        // Get DataTable parameters safely
        $limit = $request->input('length', 10);  // Default limit
        $start = $request->input('start', 0);
        $orderIndex = $request->input('order.0.column', 0);
        $order = $colOrder[$orderIndex] ?? 'book_no';  // Fallback to 'book_no' if invalid
        $dir = $request->input('order.0.dir', 'asc');
            
        // Base query
        $query = LibraryBook::query()
            ->leftJoin('library_categories', 'library_books.category_id', '=', 'library_categories.id')
            ->leftJoin('library_subcategories', 'library_books.subcategory_id', '=', 'library_subcategories.id')
            ->leftJoin('library_locations', 'library_books.location_id', '=', 'library_locations.id')
            ->leftJoin('library_sublocations', 'library_books.sublocation_id', '=', 'library_sublocations.id')
            ->leftJoinSub(
                DB::table('library_issues')
                    ->select('book_id', DB::raw('COUNT(*) as issued_count'))
                    ->where('status', 'Issued')
                    ->groupBy('book_id'),
                'issued_counts',
                'library_books.id','=','issued_counts.book_id'
            )
            ->whereNull('library_books.deleted_at');
            
        // Dynamic filters
        $filters = [
            'library_books.category_id' => $request->get('category'),
            'library_books.subcategory_id' => $request->get('subcategory'),
            'library_books.location_id' => $request->get('location'),
            'library_books.sublocation_id' => $request->get('sublocation'),
        ];
            
        foreach ($filters as $field => $value) {
            if (!empty($value)) {
                $query->where($field, $value);
            }
        }
            
        // Keyword search
        if ($skey = $request->get('skey')) {
            $query->where(function ($qry) use ($skey) {
                $qry->where('library_books.book_no', 'like', "%$skey%")
                    ->orWhere('library_books.access_no', 'like', "%$skey%")
                    ->orWhere('library_books.title', 'like', "%$skey%")
                    ->orWhere('library_books.author', 'like', "%$skey%")
                    ->orWhere('library_books.subject', 'like', "%$skey%")
                    ->orWhere('library_books.keywords', 'like', "%$skey%")
                    ->orWhere('library_categories.name', 'like', "%$skey%")
                    ->orWhere('library_subcategories.name', 'like', "%$skey%")
                    ->orWhere('library_locations.name', 'like', "%$skey%")
                    ->orWhere('library_sublocations.name', 'like', "%$skey%");
            });
        }
            
        // Search box filtering
        if ($search = $request->input('search.value')) {
            $query->where(function ($qry) use ($search) {
                $qry->where('library_books.book_no', 'like', "%$search%")
                    ->orWhere('library_books.access_no', 'like', "%$search%")
                    ->orWhere('library_books.title', 'like', "%$search%")
                    ->orWhere('library_books.author', 'like', "%$search%")
                    ->orWhere('library_books.subject', 'like', "%$search%")
                    ->orWhere('library_books.keywords', 'like', "%$search%")
                    ->orWhere('library_categories.name', 'like', "%$search%")
                    ->orWhere('library_subcategories.name', 'like', "%$search%")
                    ->orWhere('library_locations.name', 'like', "%$search%")
                    ->orWhere('library_sublocations.name', 'like', "%$search%");
            });
        }
            
        // Get total filtered count before applying pagination
        $totalFiltered = $query->count();
            
        // Select necessary columns and apply ordering, pagination
        $books = $query->select(
                'library_books.*',
                'library_categories.name as category',
                'library_subcategories.name as subcategory',
                'library_locations.name as location',
                'library_sublocations.name as sublocation',
                DB::raw('COALESCE(issued_counts.issued_count, 0) as issued_count')
            )
            ->orderBy($order, $dir)
            ->offset($start)
            ->limit($limit)
            ->get();
            
        // Permissions
        $barcode = MenuAccess::have_access(669);
        $view = MenuAccess::have_access(666);
        $edit = MenuAccess::have_access(667);
        $delete = MenuAccess::have_access(668);
            
        // Prepare data for response
        $data = [];
        foreach ($books as $book) {
            $row = [
                'bno' => htmlspecialchars($book->book_no),
                'ano' => htmlspecialchars($book->access_no),
                'title' => htmlspecialchars($book->title),
                'author' => htmlspecialchars($book->author),
                'subject' => htmlspecialchars($book->subject),
                'keyword' => htmlspecialchars($book->keywords),
                'category' => htmlspecialchars($book->category),
                'subcategory' => htmlspecialchars($book->subcategory),
                'location' => htmlspecialchars($book->location),
                'sublocation' => htmlspecialchars($book->sublocation),
                'copies' => htmlspecialchars($book->copies),
                'available' => ($book->copies-$book->issued_count),
                'action' => $this->generateActionButtons($book->id, $barcode, $view, $edit, $delete)
            ];
            $data[] = $row;
        }
            
        // Return JSON response
        return response()->json([
            'draw' => intval($request->input('draw')),
            'recordsTotal' => LibraryBook::whereNull('deleted_at')->count(),
            'recordsFiltered' => $totalFiltered,
            'data' => $data
        ]);
    }
    
    private function generateActionButtons($bookid, $barcode, $view, $edit, $delete)
    {
        $buttons = '';
        if ($barcode) {
            $buttons .= '<a href="' . route('library.book.barcode', encrypt($bookid)) . '" title="Barcode" target="_blank"><i class="icon-sm icon-frame text-dark"></i></a>&nbsp;&nbsp;&nbsp;';
        }
        if ($view) {
            $buttons .= '<a href="' . route('library.book.view', encrypt($bookid)) . '" title="View"><i class="icon-sm icon-magnifier text-info"></i></a>&nbsp;&nbsp;&nbsp;';
        }
        if ($edit) {
            $buttons .= '<a href="' . route('library.book.edit', encrypt($bookid)) . '" title="Edit"><i class="icon-sm icon-pencil text-success"></i></a>&nbsp;&nbsp;&nbsp;';
        }
        if ($delete) {
            $buttons .= '<a href="' . route('library.book.delete', encrypt($bookid)) . '" title="Delete" onclick="return confirm(\'Are you sure you want to delete?\');"><i class="icon-sm icon-trash text-danger"></i></a>';
        }
        return $buttons;
    }
    
    public function add(){
        try {
            $categories = LibraryCategory::all();
            $locations = LibraryLocation::all();
            $bno_max = LibraryBook::max('book_no');
            $bno = $bno_max ? ($bno_max+1):1000;
            
            return view('library.book.add',compact('categories','locations','bno'));
        } catch (\Exception $e) {
            Log::error('Failed to fetch Data: '.$e->getMessage());
            return back()->withErrors('Failed to load Data.');
        }
    }
    
    public function save(Request $request){
        $validated = $request->validate([
                    'access_no' => 'required',
                    'title' => 'required',
                    'sub_title' => 'nullable',
                    'author' => 'required',
                    'category_id' => 'required',
                    'subcategory_id' => 'required',
                    'location_id' => 'required',
                    'sublocation_id' => 'required',
                    'date' => 'nullable|date',
                    'language' => 'nullable',
                    'publisher' => 'nullable',
                    'volume' => 'nullable',
                    'edition' => 'nullable',
                    'year' => 'nullable|numeric',
                    'price' => 'nullable|numeric',
                    'condition' => 'required',
                    'isbn' => 'nullable',
                    'reference' => 'required',
                    'subject' => 'nullable',
                    'awards' => 'nullable',
                    'keywords' => 'nullable',
                    'writer' => 'nullable',
                    'copies' => 'required|numeric',
                    'call_no' => 'nullable',
                    'image' => 'nullable|max:5120|mimes:jpeg,png,jpg',
                ]);
        
        try{
            $image='';
            if($request->hasFile('image')){
                $ext = $request->image->extension();
                $image = 'book_'.time().'.'.$ext;
                
                Image::make($request->image)->save('public/uploads/books/image/'.$image);
            }
            
            $bno_max = LibraryBook::max('book_no');
            $bno = $bno_max ? ($bno_max+1):1000;
            
            $validated['book_no'] = $bno;
            $validated['barcode'] = '9'.rand(10000,99999).$bno;
            $validated['image'] = $image;
            
            $saved = LibraryBook::create($validated);
            if($saved){
                $msg_array = array('message'=>'Book Created Successfully !','alert-type'=>'success');
            }else{
                $msg_array = array('message'=>'Book Could not be Created !','alert-type'=>'error');
            }
        } catch (\Exception $e) {
            Log::error('Book creation failed: '.$e->getMessage());
            $msg_array = array('message'=>'Book Could not be Created !','alert-type'=>'error');
        }
            
        return redirect()->route('library.book')->with($msg_array);
    }
    
    public function import(){
        try {
            return view('library.book.import');
        } catch (\Exception $e) {
            Log::error('Failed to load Book Import Page: '.$e->getMessage());
            return back()->withErrors('Failed to load Book Import Page.');
        }
    }
    
    public function upload(Request $request){
        $validated = $request->validate([
                        'file' => 'required|file|mimes:xlsx,xls',
                    ]);
        try{
            Excel::import(new LibraryBooksImport, $request->file('file'));
                
            $msg_array = ['message' => 'Books Uploaded Successfully !', 'alert-type' => 'success'];
        } catch (\Exception $e) {
            Log::error('Book upload failed: '.$e->getMessage());
            $msg_array = array('message'=>'Book Could not be Uploaded !','alert-type'=>'error');
        }
            
        return redirect()->route('library.book')->with($msg_array);
    }
    
    public function export(Request $request){
        try {
            $category = $request->get('category') ?? 0;
            $subcategory = $request->get('subcategory') ?? 0;
            $location = $request->get('location') ?? 0;
            $sublocation = $request->get('sublocation') ?? 0;
            $skey = $request->get('skey') ?? null;
            
            return Excel::download(new LibraryBooksExport($category, $subcategory, $location, $sublocation, $skey), 'library_books.xlsx');
        } catch (\Exception $e) {
            Log::error('Error loading Books for export: '.$e->getMessage());
            return back()->withErrors('Failed to export the Books.');
        }
    }
    
    public function view($bid){
        try {
            $book = LibraryBook::findOrFail(decrypt($bid));
            
            return view('library.book.view',compact('book'));
        } catch (ModelNotFoundException $e) {
            return redirect()->route('library.book')->withErrors('book not found.');
        } catch (\Exception $e) {
            Log::error('Error loading Book for view: ' . $e->getMessage());
            return back()->withErrors('Failed to load the Book for view.');
        }
    }
    
    public function edit($bid){
        try {
            $book = LibraryBook::findOrFail(decrypt($bid));
            $categories = LibraryCategory::all();
            $subcategories = LibrarySubcategory::where('category_id',$book->category_id)->get();
            $locations = LibraryLocation::all();
            $sublocations = LibrarySublocation::where('location_id',$book->location_id)->get();
            
            return view('library.book.edit',compact('categories','subcategories','locations','sublocations','book'));
        } catch (ModelNotFoundException $e) {
            return redirect()->route('library.book')->withErrors('Book not found.');
        } catch (\Exception $e) {
            Log::error('Error loading Book for edit: '.$e->getMessage());
            return back()->withErrors('Failed to load the Book for editing.');
        }
    }
    
    public function update(Request $request, $id)
    {
        $bid = decrypt($id);
        try {
            $validated = $request->validate([
                    'access_no' => 'required',
                    'title' => 'required',
                    'sub_title' => 'nullable',
                    'author' => 'required',
                    'category_id' => 'required',
                    'subcategory_id' => 'required',
                    'location_id' => 'required',
                    'sublocation_id' => 'required',
                    'date' => 'nullable|date',
                    'language' => 'nullable',
                    'publisher' => 'nullable',
                    'volume' => 'nullable',
                    'edition' => 'nullable',
                    'year' => 'nullable|numeric',
                    'price' => 'nullable|numeric',
                    'condition' => 'required',
                    'isbn' => 'nullable',
                    'reference' => 'required',
                    'subject' => 'nullable',
                    'awards' => 'nullable',
                    'keywords' => 'nullable',
                    'writer' => 'nullable',
                    'copies' => 'required|numeric',
                    'call_no' => 'nullable',
                    'image' => 'nullable|max:5120|mimes:jpeg,png,jpg',
                ]);
                
            $book = LibraryBook::findOrFail($bid);
            
            $image = $book->image;
            if($request->hasFile('image')){
                $ext = $request->image->extension();
                $image = 'book_'.time().'.'.$ext;
                
                Image::make($request->image)->save('public/uploads/books/image/'.$image);
            }
            $validated['image'] = $image;
            
            $book->update($validated);
            
            $msg_array = array('message'=>'Book Updated Successfully !','alert-type'=>'success');
        } catch (\Exception $e) {
            Log::error('Book update failed: '.$e->getMessage());
            $msg_array = array('message'=>'Book Could not be Updated !','alert-type'=>'error');
        }
        
        return redirect()->route('library.book')->with($msg_array);
    }
    
    public function delete($id)
    {
        $bid = decrypt($id);
        try {
            $book = LibraryBook::findOrFail($bid);
            if($book->issues->where('status','Issued')->count()==0){
                $book->delete();
                $msg_array = array('message'=>'Book Deleted Successfully !','alert-type'=>'success');
            }else{
                $msg_array = array('message'=>'Book Could not be Deleted !','alert-type'=>'error');
            }
        } catch (\Exception $e) {
            Log::error('Book deletion failed: '.$e->getMessage());
            $msg_array = array('message'=>'Book Could not be Deleted !','alert-type'=>'error');
        }
        
        return redirect()->route('library.book')->with($msg_array);
    }
    
    public function barcode($bid){
        try {
            $book = LibraryBook::findOrFail(decrypt($bid));
            
            $barcodeDirectory = public_path('uploads/barcode/book/');
            $barcodeFileName = $book->barcode.'.png'; // Assuming PNG format
            $barcodeFilePath = $barcodeDirectory . $barcodeFileName;
            
            if (!File::exists($barcodeFilePath)) {
                $barcode = new DNS1D();
                $barcode->setStorPath($barcodeDirectory);
                $barcode->getBarcodePNGPath($book->barcode, 'C39',1,40,array(0,0,0), true);
            }
            
            return view('library.book.barcode',compact('book'));
        } catch (ModelNotFoundException $e) {
            return redirect()->route('library.book')->withErrors('book not found.');
        } catch (\Exception $e) {
            Log::error('Error loading Barcode for book: ' . $e->getMessage());
            return back()->withErrors('Failed to load the Barcode.');
        }
    }
    
    public function barcode_bulk(Request $request){
        try {
            $category = $request->get('category') ?? null;
            $subcategory = $request->get('subcategory') ?? null;
            $location = $request->get('location') ?? null;
            $sublocation = $request->get('sublocation') ?? null;
            $skey = $request->get('skey') ?? null;
            
            $query = LibraryBook::query()
                    ->leftJoin('library_categories', 'library_books.category_id', '=', 'library_categories.id')
                    ->leftJoin('library_subcategories', 'library_books.subcategory_id', '=', 'library_subcategories.id')
                    ->leftJoin('library_locations', 'library_books.location_id', '=', 'library_locations.id')
                    ->leftJoin('library_sublocations', 'library_books.sublocation_id', '=', 'library_sublocations.id')
                    ->whereNull('library_books.deleted_at');
            if($category){
                $query->where('library_books.category_id', $category);
            }
            if($subcategory){
                $query->where('library_books.subcategory_id', $subcategory);
            }
            if($location){
                $query->where('library_books.location_id', $location);
            }
            if($sublocation){
                $query->where('library_books.sublocation_id', $sublocation);
            }
            if($skey){
                $query->where(function ($qry) use ($skey) {
                    $qry->where('library_books.book_no', 'like', "%$skey%")
                        ->orWhere('library_books.access_no', 'like', "%$skey%")
                        ->orWhere('library_books.title', 'like', "%$skey%")
                        ->orWhere('library_books.author', 'like', "%$skey%")
                        ->orWhere('library_books.subject', 'like', "%$skey%")
                        ->orWhere('library_books.keywords', 'like', "%$skey%")
                        ->orWhere('library_categories.name', 'like', "%$skey%")
                        ->orWhere('library_subcategories.name', 'like', "%$skey%")
                        ->orWhere('library_locations.name', 'like', "%$skey%")
                        ->orWhere('library_sublocations.name', 'like', "%$skey%");
                });
            }
        
            $books = $query->select('library_books.*')->get();
            
            $barcodeDirectory = public_path('uploads/barcode/book/');
            foreach($books as $book){
                $barcodeFileName = $book->barcode.'.png';
                $barcodeFilePath = $barcodeDirectory.$barcodeFileName;
                
                if (!File::exists($barcodeFilePath)) {
                    $barcode = new DNS1D();
                    $barcode->setStorPath($barcodeDirectory);
                    $barcode->getBarcodePNGPath($book->barcode, 'C39',1,40,array(0,0,0), true);
                }
            }
            
            return view('library.book.bulk_barcode',compact('books'));
        } catch (\Exception $e) {
            Log::error('Error loading Barcode for books: '.$e->getMessage());
            return back()->withErrors('Failed to load the Barcode.');
        }
    }
}
