<?php

namespace App\Http\Controllers\Library;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;

use App\Models\Library\LibraryReservation;
use App\Models\Library\LibraryMemberType;
use App\Models\Library\LibraryIssue;
use App\Models\MenuAccess;

class Reservations extends Controller
{
    public function report(){
        $types = LibraryMemberType::all();
        
        return view('library.reservation.report',compact('types'));
    }
    
    public function getData(Request $request){
            
        $colOrder = ['membership_no', 'member_name', 'member_type', 'access_no', 'title', 'author', 'date', 'approval'];
            
        $limit = $request->input('length', 10);
        $start = $request->input('start', 0);
        $orderIndex = $request->input('order.0.column', 0);
        $order = $colOrder[$orderIndex] ?? 'id';
        $dir = $request->input('order.0.dir', 'asc');
        
        $mtype = $request->get('mtype') ?? null;
        $member = $request->get('member') ?? null;
        $book = $request->get('book') ?? null;
        $status = $request->get('status') ?? null;
        $fdate = $request->get('fdate') ?? null;
        $tdate = $request->get('tdate') ?? null;
            
        // Base query
        $query = LibraryReservation::query()
            ->leftJoin('library_memberships', 'library_reservations.member_id', '=', 'library_memberships.id')
            ->leftJoin('library_books', 'library_reservations.book_id', '=', 'library_books.id')
            ->leftJoin('library_member_types', 'library_reservations.type_id', '=', 'library_member_types.id')
            ->whereNull('library_reservations.deleted_at');
            
        if($mtype){
            $query->where('library_reservations.type_id',$mtype);
        }
        if($member){
            $query->where('library_reservations.member_id',$member);
        }
        if($book){
            $query->where('library_reservations.book_id',$book);
        }
        if($status){
            $query->where('library_reservations.approval',$status);
        }
        if($fdate){
            $query->where('library_reservations.date','>=',$fdate);
        }
        if($tdate){
            $query->where('library_reservations.date','<=',$tdate);
        }
            
        // Search box filtering
        if ($search = $request->input('search.value')) {
            $query->where(function ($qry) use ($search) {
                $qry->where('library_memberships.membership_no', 'like', "%$search%")
                    ->orWhere('library_memberships.member_name', 'like', "%$search%")
                    ->orWhere('library_member_types.member_type', 'like', "%$search%")
                    ->orWhere('library_books.access_no', 'like', "%$search%")
                    ->orWhere('library_books.title', 'like', "%$search%")
                    ->orWhere('library_books.author', 'like', "%$search%")
                    ->orWhere('library_reservations.date', 'like', "%$search%")
                    ->orWhere('library_reservations.approval', 'like', "%$search%");
            });
        }
            
        // Get total filtered count before applying pagination
        $totalFiltered = $query->count();
            
        // Select necessary columns and apply ordering, pagination
        $reservations = $query->select(
                'library_reservations.*',
                'library_memberships.membership_no',
                'library_memberships.member_name',
                'library_member_types.member_type',
                'library_books.access_no',
                'library_books.title',
                'library_books.author')
            ->orderBy($order, $dir)
            ->offset($start)
            ->limit($limit)
            ->get();
            
        $approve = MenuAccess::have_access(697);
        $reject = MenuAccess::have_access(698);
            
        // Prepare data for response
        $data = [];
        foreach ($reservations as $reservation) {
            $row = [
                'mno' => htmlspecialchars($reservation->membership_no),
                'member' => htmlspecialchars($reservation->member_name),
                'mtype' => htmlspecialchars($reservation->member_type),
                'ano' => htmlspecialchars($reservation->access_no),
                'book' => htmlspecialchars($reservation->title),
                'author' => htmlspecialchars($reservation->author),
                'date' => $reservation->date ? date('d/m/Y',strtotime($reservation->date)):'',
                'status' => htmlspecialchars($reservation->approval),
                'action' => $reservation->approval=='Pending' ? $this->generateActionButtons($reservation->id, $approve, $reject):''
            ];
            $data[] = $row;
        }
            
        // Return JSON response
        return response()->json([
            'draw' => intval($request->input('draw')),
            'recordsTotal' => LibraryReservation::whereNull('deleted_at')->count(),
            'recordsFiltered' => $totalFiltered,
            'data' => $data
        ]);
    }
    
    private function generateActionButtons($reservationid, $approve, $reject)
    {
        $buttons = '';
        if ($approve) {
            $buttons .= '<a href="' . route('library.reservation.approve', encrypt($reservationid)) . '" title="Approve"><i class="icon-sm icon-check text-success"></i></a>&nbsp;&nbsp;&nbsp;';
        }
        if ($reject) {
            $buttons .= '<a href="' . route('library.reservation.reject', encrypt($reservationid)) . '" title="Reject"><i class="icon-sm icon-close text-danger"></i></a>&nbsp;&nbsp;&nbsp;';
        }
        return $buttons;
    }
    
    public function approve($id)
    {
        $bid = decrypt($id);
        try {
            $reservation = LibraryReservation::findOrFail($bid);
            if($reservation->update(['approval'=>'Approved'])){
                LibraryIssue::create([
                        'type_id' => $reservation->type_id,
                        'member_id' => $reservation->member_id,
                        'book_id' => $reservation->book_id,
                    ]);
                
                $msg_array = array('message'=>'Reservation Approved Successfully !','alert-type'=>'success');
            }else{
                $msg_array = array('message'=>'Reservation Could not be Approved !','alert-type'=>'error');
            }
        } catch (\Exception $e) {
            Log::error('Reservation approval failed: '.$e->getMessage());
            $msg_array = array('message'=>'Reservation Could not be Approved !','alert-type'=>'error');
        }
        
        return redirect()->route('library.reservation')->with($msg_array);
    }
    
    public function reject($id)
    {
        $bid = decrypt($id);
        try {
            $reservation = LibraryReservation::findOrFail($bid);
            if($reservation->update(['approval'=>'Rejected'])){
                $msg_array = array('message'=>'Reservation Rejected Successfully !','alert-type'=>'success');
            }else{
                $msg_array = array('message'=>'Reservation Could not be Rejected !','alert-type'=>'error');
            }
        } catch (\Exception $e) {
            Log::error('Reservation rejection failed: '.$e->getMessage());
            $msg_array = array('message'=>'Reservation Could not be Rejected !','alert-type'=>'error');
        }
        
        return redirect()->route('library.reservation')->with($msg_array);
    }
}
