<?php

namespace App\Http\Controllers\Student;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\File;

use App\Models\Library\LibraryMembership;
use App\Models\Library\LibraryReservation;
use App\Models\Library\LibraryBook;
use App\Models\Library\LibraryCategory;

use DB;

class Library extends Controller
{
    
    public function reservation(){
        $studentId = Auth::guard('student')->id();
        $membership = LibraryMembership::where('membership_type',2)->where('membership_user',$studentId)->where('status','Active')->first();
        
        return view('student.library.reservation',compact('membership'));
    }
    
    public function getReservations(Request $request){
            
        $colOrder = ['access_no', 'title', 'author', 'date', 'approval'];
            
        $limit = $request->input('length', 10);
        $start = $request->input('start', 0);
        $orderIndex = $request->input('order.0.column', 0);
        $order = $colOrder[$orderIndex] ?? 'id';
        $dir = $request->input('order.0.dir', 'asc');
        
        $fdate = $request->get('fdate') ?? null;
        $tdate = $request->get('tdate') ?? null;
        
        $studentId = Auth::guard('student')->id();
        $membership = LibraryMembership::where('membership_type',2)->where('membership_user',$studentId)->where('status','Active')->first();
            
        // Base query
        $query = LibraryReservation::query()
            ->leftJoin('library_books', 'library_reservations.book_id', '=', 'library_books.id')
            ->where('library_reservations.member_id',$membership->id);
            
        if($fdate){
            $query->where('library_reservations.date','>=',$fdate);
        }
        if($tdate){
            $query->where('library_reservations.date','<=',$tdate);
        }
            
        // Search box filtering
        if ($search = $request->input('search.value')) {
            $query->where(function ($qry) use ($search) {
                $qry->where('library_books.access_no', 'like', "%$search%")
                    ->orWhere('library_books.title', 'like', "%$search%")
                    ->orWhere('library_books.author', 'like', "%$search%")
                    ->orWhere('library_reservations.date', 'like', "%$search%")
                    ->orWhere('library_reservations.approval', 'like', "%$search%");
            });
        }
            
        // Get total filtered count before applying pagination
        $totalFiltered = $query->count();
            
        // Select necessary columns and apply ordering, pagination
        $reservations = $query->select(
                'library_reservations.*',
                'library_books.access_no',
                'library_books.title',
                'library_books.author')
            ->orderBy($order, $dir)
            ->offset($start)
            ->limit($limit)
            ->get();
            
        // Prepare data for response
        $data = [];
        foreach ($reservations as $reservation) {
            $row = [
                'ano' => htmlspecialchars($reservation->access_no),
                'book' => htmlspecialchars($reservation->title),
                'author' => htmlspecialchars($reservation->author),
                'date' => $reservation->date ? date('d/m/Y',strtotime($reservation->date)):'',
                'status' => htmlspecialchars($reservation->approval),
                'action' => $reservation->approval=='Pending' ? $this->generateActionButtons($reservation->id):''
            ];
            $data[] = $row;
        }
            
        // Return JSON response
        return response()->json([
            'draw' => intval($request->input('draw')),
            'recordsTotal' => LibraryReservation::where('library_reservations.member_id',$membership->id)->count(),
            'recordsFiltered' => $totalFiltered,
            'data' => $data
        ]);
    }
    
    private function generateActionButtons($reservationid)
    {
        $buttons .= '<a href="' . route('student.reservation.cancel', encrypt($reservationid)) . '" title="Cancel"><i class="icon-sm icon-close text-danger"></i></a>';
        
        return $buttons;
    }
    
    public function add_reservation(){
        try {
            $studentId = Auth::guard('student')->id();
            $membership = LibraryMembership::where('membership_type',2)->where('membership_user',$studentId)->where('status','Active')->first();
            $categories = LibraryCategory::all();
            
            return view('student.library.add_reservation',compact('membership','categories'));
        } catch (\Exception $e) {
            Log::error('Failed to fetch Books: '.$e->getMessage());
            return back()->withErrors('Failed to load Books.');
        }
    }
    
    public function books(Request $request){
            
        $colOrder = ['access_no', 'title', 'author', 'category', 'subcategory'];
            
        // Get DataTable parameters safely
        $limit = $request->input('length', 10);  // Default limit
        $start = $request->input('start', 0);
        $orderIndex = $request->input('order.0.column', 0);
        $order = $colOrder[$orderIndex] ?? 'id'; 
        $dir = $request->input('order.0.dir', 'asc');
            
        // Base query
        $query = LibraryBook::query()
            ->leftJoin('library_categories', 'library_books.category_id', '=', 'library_categories.id')
            ->leftJoin('library_subcategories', 'library_books.subcategory_id', '=', 'library_subcategories.id')
            ->leftJoinSub(
                DB::table('library_issues')
                    ->select('book_id', DB::raw('COUNT(*) as issued_count'))
                    ->where('status', 'Issued')
                    ->groupBy('book_id'),
                'issued_counts',
                'library_books.id','=','issued_counts.book_id'
            )
            ->whereNull('library_books.deleted_at');
            
        // Dynamic filters
        $filters = [
            'library_books.category_id' => $request->get('category'),
            'library_books.subcategory_id' => $request->get('subcategory'),
        ];
            
        foreach ($filters as $field => $value) {
            if (!empty($value)) {
                $query->where($field, $value);
            }
        }
            
        // Keyword search
        if ($skey = $request->get('skey')) {
            $query->where(function ($qry) use ($skey) {
                $qry->where('library_books.access_no', 'like', "%$skey%")
                    ->orWhere('library_books.title', 'like', "%$skey%")
                    ->orWhere('library_books.author', 'like', "%$skey%")
                    ->orWhere('library_categories.name', 'like', "%$skey%")
                    ->orWhere('library_subcategories.name', 'like', "%$skey%");
            });
        }
            
        // Search box filtering
        if ($search = $request->input('search.value')) {
            $query->where(function ($qry) use ($search) {
                $qry->where('library_books.access_no', 'like', "%$search%")
                    ->orWhere('library_books.title', 'like', "%$search%")
                    ->orWhere('library_books.author', 'like', "%$search%")
                    ->orWhere('library_categories.name', 'like', "%$search%")
                    ->orWhere('library_subcategories.name', 'like', "%$search%");
            });
        }
            
        // Get total filtered count before applying pagination
        $totalFiltered = $query->count();
            
        // Select necessary columns and apply ordering, pagination
        $books = $query->select(
                'library_books.*',
                'library_categories.name as category',
                'library_subcategories.name as subcategory',
                DB::raw('COALESCE(issued_counts.issued_count, 0) as issued_count')
            )
            ->orderBy($order, $dir)
            ->offset($start)
            ->limit($limit)
            ->get();
            
        // Prepare data for response
        $data = [];
        foreach ($books as $book) {
            $row = [
                'ano' => htmlspecialchars($book->access_no),
                'title' => htmlspecialchars($book->title),
                'author' => htmlspecialchars($book->author),
                'category' => htmlspecialchars($book->category),
                'subcategory' => htmlspecialchars($book->subcategory),
                'available' => ($book->copies-$book->issued_count),
                'action' => '',//$this->generateActionButtons($book->id)
            ];
            $data[] = $row;
        }
            
        // Return JSON response
        return response()->json([
            'draw' => intval($request->input('draw')),
            'recordsTotal' => LibraryBook::whereNull('deleted_at')->count(),
            'recordsFiltered' => $totalFiltered,
            'data' => $data
        ]);
    }
}
